/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// -- Namespaces ------------------------------------------------------------
// #include aura.AuraNamespaces

// -- Aura Bootstrap ------------------------------------------------------------
Aura.time = window.performance && window.performance.now ? window.performance.now.bind(performance) : function(){};
Aura["bootstrap"] = Aura["bootstrap"] || {};
Aura["bootstrap"]["visibilityStateStart"] = document.visibilityState;
Aura.bootstrapMark = function (mark, value) {};


// bootstrap robustness:
// - all files required for bootstrap are loaded with sync <script> tags
// - DOMContentLoaded is fired after DOM is ready, which includes executing all sync <script> tags
// - to detect errors we wait for DOMContentLoaded then check for all bootstrap file markers
// NOTE: if Aura is injected into the page (Lightning Out or AIS) then DOMContentLoaded is fired
// before aura.js is loaded/executed, so verifyBootstrap() is never invoked.
//#if {"excludeModes" : ["DOC","TESTING","AUTOTESTING", "TESTINGDEBUG", "AUTOTESTINGDEBUG"]}
(function bootstrapRobustness() {})();
//#end

function noop() {}

// This, $A, is supposed to be our ONLY window-polluting top-level variable.
// Everything else in Aura is attached to it.
// window['$A'] = {};

// -- Polyfills --------------------------------------------------------
// #include aura.polyfill.Json
// #include aura.polyfill.TextEncoder
// #include aura.polyfill.stackframe
// #include aura.polyfill.error-stack-parser

// -- Exporter ---------------------------------------------------------
// #include aura.util.ExportSymbolsHelper

// -- Context -----------------------------------------------------------
// #include aura.context.AuraContext

// -- Utils ------------------------------------------------------------
// #include aura.util.Style
// #include aura.util.Bitset
// #include aura.util.Duration
// #include aura.util.NumberFormat
// #include aura.util.Mutex
// #include aura.util.DocLevelHandler
// #include aura.util.SizeEstimator
// #include aura.util.SecureFilters
// #include aura.util.Util

// #include aura.Logger
// #include aura.util.Override

// -- Errors ------------------------------------------------------------
// #include aura.error.AuraError
// #include aura.error.AuraFriendlyError

// -- System ------------------------------------------------------------
// #include aura.system.DefDescriptor

// -- Style -------------------------------------------------------------
// #include aura.style.StyleDef

// -- Flavors -----------------------------------------------------------
// #include aura.flavors.FlavorDefaultDef
// #include aura.flavors.FlavorsDef

// -- Value -------------------------------------------------------------
// #include aura.value.PropertyReferenceValue
// #include aura.value.FunctionCallValue
// #include aura.value.PassthroughValue
// #include aura.value.ValueFactory
// #include aura.value.ExpressionFunctions

// -- Model -------------------------------------------------------------
// #include aura.model.ValueDef
// #include aura.model.ModelDef
// #include aura.model.Model

// -- Component ---------------------------------------------------------
// #include aura.component.ComponentClassRegistry
// #include aura.component.ComponentDefLoader
// #include aura.component.ComponentDefStorage
// #include aura.component.Component
// #include aura.component.BaseComponent
// #include aura.component.ExpressionComponent
// #include aura.component.HtmlComponent
// #include aura.component.IfComponent
// #include aura.component.IterationComponent
// #include aura.component.TextComponent
// #include aura.component.InteropComponent
// #include aura.component.InteropModule
// #include aura.component.ComponentDef
// #include aura.component.InteropComponentDef
// #include aura.component.ActionValueProvider
// #include aura.component.EventValueProvider
// #include aura.component.StyleValueProvider

// -- Renderer ----------------------------------------------------------

// -- Provider ----------------------------------------------------------
// #include aura.provider.GlobalValueProviders
// #include aura.provider.LabelQueue
// #include aura.provider.LabelValueProvider
// #include aura.provider.ObjectValueProvider
// #include aura.provider.ContextValueProvider

// -- Helper -------------------------------------------------------------

// -- Library ------------------------------------------------------------
// #include aura.library.LibraryIncludeRegistry
// #include aura.library.LibraryRegistry

// -- Event --------------------------------------------------------------
// #include aura.event.EventDef
// #include aura.event.Event
// #include aura.event.InteropEvent

// -- Controller ---------------------------------------------------------
// #include aura.controller.ActionDef
// #include aura.controller.Action
// #include aura.controller.ActionStorage
// #include aura.controller.ControllerDef

// -- Attribute ----------------------------------------------------------
// #include aura.attribute.AttributeDef
// #include aura.attribute.AttributeSet
// #include aura.attribute.AttributeDefSet

// -- Method ----------------------------------------------------------
// #include aura.method.MethodDef

// -- RequiredVersion ----------------------------------------------------------
// #include aura.requiredVersion.RequiredVersionDef
// #include aura.requiredVersion.RequiredVersionDefSet

// -- Storage -------------------------------------------------------------
// #include aura.storage.AuraStorageService
// #include aura.storage.AuraStorage

// -- Services -----------------------------------------------------------
// #include aura.AuraClientService
// #include aura.AuraComponentContext
// #include aura.AuraComponentService
// #include aura.AuraRenderingService
// #include aura.AuraExpressionService
// #include aura.AuraHistoryService
// #include aura.AuraEventService
// #include aura.AuraLocalizationService
// #include aura.AuraStyleService
// #include aura.metrics.AuraMetricsService

//-- Module Exports ----------------------------------------------------------
// #include aura.modules.AuraExportsServiceApi
// #include aura.modules.AuraExportsCompat
// #include aura.modules.AuraExportsStorage
// #include aura.modules.AuraExportsLogger
// #include aura.modules.AuraExportsAssert
// #include aura.modules.AuraExportsMetricsService

// -- Mode injection ---------------------------------------------------------
// #include {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"], "path" : "aura.AuraDevToolService"}

//-- LockerService -----------------------------------------------------------
//#include aura.lockerservice.LockerService

// -- $A Instance ------------------------------------------------------------
// #include aura.AuraInstance

// At this point, Aura has been defined with all our types on it, but $A itself
// is just a placeholder. Use this function to preserve Aura while populating
// $A, without making a new top-level name:
(function bootstrap() {})();

// TODO: Remove the legacy 'aura' top-level name.
window['aura'] = window['$A'];

// -- Storage Adapters -------------------------------------------------
// #include aura.storage.adapters.MemoryAdapter
// #include aura.storage.adapters.IndexedDBAdapter
// #include aura.storage.adapters.CryptoAdapter

// -- Metrics Plugins --------------------------------------------------
// #include aura.metrics.plugins.TransportMetricsPlugin
// #include aura.metrics.plugins.PerfMetricsPlugin
// #include aura.metrics.plugins.QueuedActionsMetricsPlugin
// #include aura.metrics.plugins.AuraContextPlugin
// #include aura.metrics.plugins.DomHandlersPlugin
// #include aura.metrics.plugins.ComponentMetricsPlugin

// #include {"excludeModes" : ["PRODUCTION"], "path" : "aura.metrics.plugins.ComponentServiceMetricsPlugin"}
// #include aura.Logging
// #include {"modes" : ["DOC","TESTING","AUTOTESTING", "TESTINGDEBUG", "AUTOTESTINGDEBUG"], "path" : "aura.test.Test"}

/*
 * Async Bootstrap dependency
 * The scripts are loaded with no order, which means we need to call initAsync in case it was already evaluated.
 * TODO W-??? change bootstrap scripts to be async+defered for perf gains, after which this code block will be required
*/

Aura["frameworkJsReady"] = true;

// This should be similar to initFramework
if (Aura["initConfig"]) {
    // LockerService must be initialized before scripts can be executed.
    $A.lockerService.initialize(Aura["initConfig"]["context"]);

    var scripts = Aura["inlineJsLocker"];
    if (scripts) {
        for (var i = 0; i < scripts.length; i++) {
            $A.lockerService.runScript(scripts[i]["callback"], scripts[i]["namespace"]);
        }
        delete Aura["inlineJsLocker"];
    }

    setTimeout(function () {}, 0);
}

// External libraries (like moment.js) will be appended here
