/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @description The Aura Expression Service, accessible using <code>$A.expressionService</code>.
 * Processes expressions.
 * @constructor
 * @export
 */
function AuraExpressionService() {}

AuraExpressionService.PRIMITIVE_SEPARATOR = "__";
AuraExpressionService.AURA_IF = "aura:if";
AuraExpressionService.AURA_ITERATION = "aura:iteration";

AuraExpressionService.prototype.getReference = function (expression, valueProvider) {};

AuraExpressionService.prototype.clearReferences=function(valueProvider){};

// JBUCH: TODO: THIS WAS FIRST ATTEMPT AT UNIFYING PRVs
//AuraExpressionService.prototype.updateReference = function (expression, valueProvider) {
//    expression = $A.expressionService.normalize(expression);
//    var reference=null;
//    if($A.util.isComponent(valueProvider)){
//        var target=this.references[valueProvider.getGlobalId()];
//        reference=target&&target[expression];
//    }else{
//        reference=this.references[expression];
//    }
//    if(reference){
//        for(var consumer in reference.consumers){
//            var component=$A.getComponent(consumer);
//            for(var targetExpression in reference.consumers[consumer]){
//                component.markDirty(targetExpression);
//            }
//        }
//    }
//};
//
//AuraExpressionService.prototype.updateReferences = function (expression, valueProvider) {
//    expression = $A.expressionService.normalize(expression);
//    var reference=null;
//    if($A.util.isComponent(valueProvider)){
//        var target=this.references[valueProvider.getGlobalId()];
//        reference=target&&target[expression];
//    }else{
//        reference=this.references[expression];
//    }
//    if(reference){
//        for(var consumer in reference.consumers){
//            var component=$A.getComponent(consumer);
//            for(var targetExpression in reference.consumers[consumer]){
//                component.markDirty(targetExpression);
//            }
//        }
//    }
//};

//JBUCH: TODO: FIXME: HACK
AuraExpressionService.prototype.updateGlobalReference = function (expression, oldValue, value) {};

AuraExpressionService.prototype.updateGlobalReferences = function (type, newValues) {};

AuraExpressionService.prototype.addExpressionListener = function (reference, expression, valueProvider) {};

AuraExpressionService.prototype.removeExpressionListener = function (reference, expression, valueProvider) {};
/**
 * @export
 */
AuraExpressionService.prototype.create = function(valueProvider, config) {};

/**
 * @export
 */
    // TODO: unify with above create method
AuraExpressionService.prototype.createPassthroughValue = function(primaryProviders, cmp) {};


/**
 * Trims markup syntax off a given string expression, removing
 * expression notation, and array notation.
 *
 * @param {Object}
 *            expression The expression to be normalized.
 * @returns {Object} The normalized string, or the input parameter, if
 *          it was not a string.
 * @export
 */
AuraExpressionService.prototype.normalize = function(expression) {};

/**
 * Resolves a hierarchical dot expression in string form against the
 * provided object if possible.
 *
 * @param {String}
 *            expression The string expression to be resolved.
 * @param {Object}
 *            container The object against which to resolve the
 *            expression.
 * @param {Boolean}
 *            rawValue Whether or not to evaluate expressions.
 * @returns {Object} The target of the expression, or undefined.
 * @export
 */
AuraExpressionService.prototype.resolve = function(expression, container, rawValue) {};

/**
 * @param cmp - component
 * @param locatorDef - LocatorDef defined in cmp
 *
 * Resolves values within a locatorContext inside a locatorDef.
 * @private
 */
AuraExpressionService.prototype.resolveLocatorContext = function (cmp, locatorDef) {};

/**
 * Returns the component that cmp is contained in
 * @param cmp - component
 *
 * @returns The component that contains cmp, bypassing if/iteration in the chain
 */
AuraExpressionService.prototype.getContainer = function (cmp) {};

/**
 * Returns a locatorDef targeting targetId inside cmp or it's super chain
 * @param cmp - component
 * @param targetId - targetId of locator def
 *
 * @returns Any locator definition found in the super chain of a component
 * @private
 */
AuraExpressionService.prototype.findLocatorDefInSuperChain = function (cmp, targetId) {};

/**
 * @param component The component that contains the locator targeting root
 * @param root Starting point of component hierarchy that requires resolving
 * @param includeMetadata Log additional metadata about the component name
 * @param primitiveFound The ID of the primitive if any that's been found
 * @returns This will produce a locator that combines information as such:
 *          + root localId as target with context provided by parent->root locator
 *          + parent localId as scope with context provided by grandparent->parent locator
 */
AuraExpressionService.prototype.resolveLocator = function (parent, root, includeMetadata, primitiveFound) {};

Aura.Services.AuraExpressionService = AuraExpressionService;
