/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The Aura History Service, accessible using <code>$A.historyService</code>. Manages browser history.
 * Internet Explorer 7 and 8 are not supported for this service.
 * @constructor
 * @export
 */
function AuraHistoryService() {}

/**
 * Sets the new location. For example, <code>set("search")</code> sets the location to <code>#search</code>.
 *
 * For browsers that don't support pushState (eg native Android browser, iOS7 webview), a manual hash
 * update and firing onhashchange is used.
 *
 * @param {Object} token The provided token set to the current location hash
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.set = function(token) {};

/**
 * Replaces the current location with the new location, meaning the current location will not be
 * stored in the browser history. Analogous to window.history.replace().
 * For example, if the last location in history was '#first', the current location is '#second',
 * then <code>replace("third")</code> replaces <code>#second</code> with <code>#third</code>
 * as the current location. Pressing the browser back button would result in location <code>#first</code>
 *
 * For browsers that don't support pushState or location.replace (eg native Android browser, iOS7 webview),
 * workarounds are used.
 *
 * @param {Object} token The provided token set to the current location hash
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.replace = function(token) {};

/**
 * Parses the location. A token can be used here.
 * <p>Example:</p>
 * <code>token == "newLayout";<br /> $A.historyService.get().token;</code>
 *
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.get = function() {};

/**
 * Loads the previous URL in the history list. Standard JavaScript <code>history.go()</code> method.
 *
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.back = function() {};

/**
 * Sets the title of the document.
 *
 * @param {String} title The new title
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.setTitle = function(title) {};

/**
 * Loads the next URL in the history list. Standard JavaScript <code>history.go()</code> method.
 *
 * @memberOf AuraHistoryService
 * @public
 * @export
 */
AuraHistoryService.prototype.forward = function() {};

/**
 * Resets history
 *
 * @public
 * @export
 */
AuraHistoryService.prototype.reset = function () {};

/**
 * Whether to use pushState.
 * Native Android browser has issues with pushState
 * IOS7 UIWebView has issues with pushState and history
 * @returns {boolean} true if pushState should be used
 * @private
 */
AuraHistoryService.prototype.usePushState = function() {};

/**
 * @private
 */
AuraHistoryService.prototype.init = function() {};

/**
 * @private
 */
AuraHistoryService.prototype.getEvent = function(){};

/**
 * @private
 */
AuraHistoryService.prototype.changeHandler = function(){};

/**
 * @private
 * @param {String} location the hash portion of the URL
 */
AuraHistoryService.prototype.parseLocation = function(location) {};


/**
 * Parses a query string into an object. It supports values in query string containing '='.
 * @param {String} querystring the query string part of the URL, the portion after '?'.
 * @private
 */
AuraHistoryService.prototype.parseQueryString = function(querystring) {};

/**
 * Firefox unescapes location.hash, so the results are inconsistent browser to browser.
 * This method normalizes accessing the hash portion.
 * @private
 */
AuraHistoryService.prototype.getLocationHash = function() {};

/**
 * @private
 */
AuraHistoryService.prototype.getHistoryState = function(){};

Aura.Services.AuraHistoryService = AuraHistoryService;
