/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*eslint-disable no-console */

/**
 * Logger implementation providing log messages to subscribers
 *
 * Subscriptions are made individually to each log level
 *
 * @namespace
 * @constructor
 * @export
 */
function Logger() {}

/**
 * Info log
 *
 * @param {String} info message
 * @param {Error} [error] error
 * @export
 */
Logger.prototype.info = function(info, error) {};

/**
 * Warning log
 *
 * @param {String} warning message
 * @param {Error} [error] error
 * @export
 */
Logger.prototype.warning = function(warning, error) {};

/**
 * Checks condition and logs message when condition is falsy
 *
 * @param {Boolean} condition check
 * @param {String} assertMessage message when assertion fails
 */
Logger.prototype.logAssert = function(condition, assertMessage) {};

/**
 * Error log and shows dialog window with error message. Also displays stack unless in production
 *
 * @param {String} msg error message
 * @param {Error} [e] error
 */
Logger.prototype.logError = function(msg, e) {};

/**
 * Report an error to the server to be logged.
 *
 * @param {AuraError} e exception to report upon.
 * @param {Action} [action] the action being performed when the exception occurred.
 * @param {string} [level] error reporting level. The default value is "ERROR". Options: ["INFO", "WARNING", "ERROR"]
 * @param {boolean} [foreground] don't set the report action as a caboose, should only be used for catastrophic failures where no futher actions will be called.
 * @private
 */
Logger.prototype.reportError = function(e, action, level, foreground) {};

/**
 * Check if an error is external. 'external' means every stackframe isn't from framework nor framework consumers.
 *
 * @param {Error} e - The error object to check
 */
Logger.prototype.isExternalError = function(e) {};

/**
 * Check if an error is raised from external code
 *
 * @param {Error} e - The error object to check
 */
Logger.prototype.isExternalRaisedError = function(e) {};

/**
 * Generate stack frames from an error.
 *
 * @private
 */
Logger.prototype.generateStackFrames = function(e) {};

/**
 * Check if a file belongs to Aura file. The file name string needs to be full path.
 *
 * @private
 */
Logger.prototype.isAuraFile = function(fileName) {};

/**
 * Checks for subscribers and notifies
 *
 * @param {String} level log level
 * @param {String} message log message
 * @param {Error} [error]
 */
Logger.prototype.log = function(level, message, error) {};

/**
 * Loops through subscribers and applies arguments to provider callback
 *
 * @param {String} level log level
 * @param {String} message log message
 * @param {Error} [error]
 */
Logger.prototype.notify = function(level, msg, error) {};

/**
 * Returns the stack trace, including the functions on the stack if available (Error object varies across browsers).
 * Values are not logged.
 *
 * @param {Error} e error
 * @param {Number} [remove]
 * @returns {String|null} stack
 */
Logger.prototype.getStackTrace = function(e, remove) {};

/**
 * Stringify a log message.
 *
 * @param {String} logMsg message
 * @param {Error} error
 * @param {Array} trace
 * @returns {String} string log
 */
Logger.prototype.stringVersion = function(logMsg, error, trace) {};

/**
 * Adds subscriber. Callback function will be called when log of level specified occurs.
 * Each level requires a subscription.
 *
 * @param {String} level log level
 * @param {Function} callback function
 * @export
 */
Logger.prototype.subscribe = function(level, callback) {};

/**
 * Removes subscription. Each level needs to be unsubscribed separately
 *
 * @param {String} level log level
 * @param {Function} callback function
 * @export
 */
Logger.prototype.unsubscribe = function(level, callback) {};

/**
 * Checks whether level is valid
 * @param {String} level log level
 * @returns {boolean}
 */
Logger.prototype.isValidLevel = function(level) {};

/**
 * Checks and throws Error if not valid subscriber
 *
 * @param {String} level log level
 * @param {Function} callback function
 * @throws Throws an error if the level is not valid or callback is not a function.
 */
Logger.prototype.validateSubscriber = function(level, callback) {};

/**
 * Returns number of subscriptions for given level
 *
 * @param {String} level
 * @returns {boolean} Whether there are subscriptions to given level
 */
Logger.prototype.hasSubscriptions = function(level) {};

//#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
/**
 * Prints log to the console (if available).
 * @private
 */
Logger.prototype.devDebugConsoleLog = function(level, message, error) {};
//#end

Aura.Utils.Logger = Logger;

/**
 * Maximum size for a stacktrace to be logged.
 *
 * @private
 */
Aura.Utils.Logger.MAX_STACKTRACE_SIZE = 25000;
