/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @description Creates an AttributeSet instance.
 * @param {Object}
 *            config Sets the values with the config object, if provided.
 * @param {Object}
 *            valueProvider Sets the value provider for the attributes.
 * @param {AttributeDefSet}
 *            attributeDefSet The metadata describing the attributes in the set.
 * @constructor
 * @protected
 */
function AttributeSet(attributeDefSet) {}

/**
 * Whether attribute exists
 *
 * @param {String}
 *            name - name of attribute
 * @returns {boolean} true if attribute exists
 * @private
 */
AttributeSet.prototype.hasAttribute = function(name) {};

/**
 * Returns the highest extended reference of the attribute using property syntax.
 *
 * @param {String} key The data key to look up.
 * @param {Object} component The component hierarchy to investigate
 * @returns {Object} the attribute def
 * @protected
 *
 */
AttributeSet.getDef = function(key, component) {};
/**
 * Returns the value referenced using property syntax.
 *
 * @param {String}
 *            key The data key to look up.
 * @returns {Object} the value of the attribute
 * @protected
 *
 */
AttributeSet.prototype.get = function(key, component) {};

/**
 * simplified version of component.get('v.body'),
 * only supposed to be used by simple components.
 *
 * @private
 *
 */
AttributeSet.prototype.getBody = function(globalId) {};

/**
 * simplified version of component.get('v.key'),
 * only supposed to be used by simple components.
 *
 * @private
 *
 */
AttributeSet.prototype.getValue = function(key) {};

AttributeSet.prototype.getShadowValue=function(key){};


AttributeSet.prototype.setShadowValue=function(key,value){};

/**
 * Set the attribute of the given name to the given value.
 *
 * @param {String}
 *            key The key can be a path expression inside. E.g.
 *            attribute.nestedValue.value....}
 * @param {Object}
 *            value The value to be set.
 *
 * @protected
 *
 */
AttributeSet.prototype.set = function(key, value, component) {};

/**
 * Clears a property reference value of the given name, and returns it. Does nothing if the attribute
 * does not exist or is not a property reference value.
 *
 * @param {String}
 *            key The key can be a path expression inside. E.g.
 *            attribute.nestedValue.value....}
 *
 * @returns {PropertyReferenceValue} the reference that was found and cleared, or null
 * @protected
 *
 */
AttributeSet.prototype.clearReference = function(key) {};

/**
 * Verifies if a value is valid for the type that the attribute is defined as.
 * Strings as strings, arrays as arrays, etc.
 */
AttributeSet.prototype.isValueValidForAttribute = function(attributeName, value) {};


AttributeSet.prototype.isTypeOfArray = function(attributeName) {};

/**
 * Reset the attribute set to point at a different def set.
 *
 * Allows us to change the set of attributes in a set when we inject a new
 * component. No checking is done here, if checking is desired, it should be
 * done by the caller.
 *
 * Doesn't check the current state of attributes because they don't matter. This
 * will create/update attributes based on new AttributeDefSet, provided
 * attribute config and current attribute values
 *
 * @param {AttributeDefSet}
 *            attributeDefSet the new def set to install.
 * @param {Object}
 *            attributes - new attributes configuration
 * @private
 */
AttributeSet.prototype.merge = function(attributes, attributeDefSet, component) {};

/**
 * Gets default attribute value.
 *
 * @param {String}
 *            name - name of attribute
 * @private
 */
AttributeSet.prototype.getDefault = function(name) {};

/**
 * Destroys the attributeset.
 *
 * @private
 */
AttributeSet.prototype.destroy = function() {};

/**
 * Loop through AttributeDefSet and create or update value using provided config
 *
 * @param {Object}
 *            config - attribute configuration
 * @private
 */
AttributeSet.prototype.initialize = function(attributes,component) {};

Aura.Attribute.AttributeSet = AttributeSet;





