/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new BaseComponent.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 */
function BaseComponent(config, localCreation) {}

BaseComponent.prototype = Object.create(Component.prototype);

// Not returning anything in these since empty functions will not be called by the JS engine as an
BaseComponent.prototype.superRender = function(){};
BaseComponent.prototype.superAfterRender = function(){};
BaseComponent.prototype.superRerender = function(){};
BaseComponent.prototype.superUnrender = function(){};
BaseComponent.prototype.getSuper = function(){};
BaseComponent.prototype.getSuperest = function(){};

BaseComponent.prototype.setupValueProviders = function(customValueProviders) {};

BaseComponent.prototype.setupComponentDef = function() {};

BaseComponent.prototype["renderer"] = {
    "render": function(component){},

    "afterRender": function(component){},

    "rerender": function(component){},

    "unrender" : function(component){}
};

Aura.Component.BaseComponent = BaseComponent;
