/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new ExpressionComponent.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 */
function ExpressionComponent(config, localCreation) {}

ExpressionComponent.prototype = Object.create(Component.prototype);

/** The SuperRender calls are blank since we will never have a super, no need to ever do any logic to for them. */
ExpressionComponent.prototype.superRender = function(){};
ExpressionComponent.prototype.superAfterRender = function(){};
ExpressionComponent.prototype.superRerender = function(){};
ExpressionComponent.prototype.superUnrender = function(){};

/** No Super, so just return undefined */
ExpressionComponent.prototype.getSuper = function(){};

/** Will always be Superest, so no need to check for a super */
ExpressionComponent.prototype.getSuperest = function(){};

ExpressionComponent.prototype.setContainerComponentId = function(containerComponentId) {};

ExpressionComponent.prototype.setupValueProviders = function(customValueProviders) {};

/**
 * Component.js has logic that is specific to HtmlComponent. Great! So we can move that into here and out of Component.js
 * That logic is the LockerService part to assign trust to the owner.
 */
ExpressionComponent.prototype.setupComponentDef = function() {};

/**
 * Simple type checking. All simple components implement aura:rootComponent and cannot be extended,
 * so the simple condition here is sufficient unless any of the individual components change.
 */
ExpressionComponent.prototype.isInstanceOf = function(type) {};


ExpressionComponent.prototype["renderer"] = {
    "render" : function(component) {},

    "rerender" : function(component) {},

    "unrender" : function(component) {},

    "afterRender" : function(component) {}
};




Aura.Component.ExpressionComponent = ExpressionComponent;
