/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new IfComponent.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 */
function IfComponent(config, localCreation) {}

IfComponent.prototype = Object.create(Component.prototype);

// Not returning anything in these since empty functions will not be called by the JS engine as an
IfComponent.prototype.setupModel = function(){};
IfComponent.prototype.superRender = function(){};
IfComponent.prototype.superAfterRender = function(){};
IfComponent.prototype.superRerender = function(){};
IfComponent.prototype.superUnrender = function(){};
IfComponent.prototype.getSuper = function(){};
IfComponent.prototype.getModel = function(){};
IfComponent.prototype.getSuperest = function(){};

IfComponent.prototype.setupComponentDef = function() {};

IfComponent.prototype.setupValueProviders = function(customValueProviders) {};

/**
 * Simple type checking. All simple components implement aura:rootComponent and cannot be extended, 
 * so the simple condition here is sufficient unless any of the individual components change.
 */
IfComponent.prototype.isInstanceOf = function(type) {};

IfComponent.prototype["controller"] = {
    "init": function(cmp, evt, helper) {},
    "handleTheTruth": function(cmp, evt, helper) {}
};


IfComponent.prototype["helper"] = {
    createBody: function(cmp, isTrue) {},

    clearUnrenderedBody: function(cmp) {}
};


IfComponent.prototype["provider"] = {
    "provide" : function(component) {}
};

// Shares the renderer
IfComponent.prototype["renderer"] = Aura.Component.BaseComponent.prototype["renderer"];

Aura.Component.IfComponent = IfComponent;
