/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @class InteropComponent
 * @constructor
 * @private
 * @export
 */
function InteropComponent(config) {}

// Prototype chain (instanceOf)
InteropComponent.prototype = Object.create(Component.prototype);
InteropComponent.prototype.constructor = InteropComponent;

InteropComponent.prototype.bridgeAction = function (prv, isEvent, hasNativeAPIExposed, action) {};

InteropComponent.prototype.hasNativeAPIExposed = function (eventName) {};


InteropComponent.prototype.setupAttributes = function(config) {};


InteropComponent.prototype.isHtmlGlobalAttr = function (attrName) {};

InteropComponent.prototype.isReadOnlyProperty = function (propName) {};

InteropComponent.HTML_GLOBAL_ATTRS = {
    'accesskey': true,
    'class': true,
    'contenteditable': true,
    'contextmenu': true,
    'dir': true,
    'draggable': true,
    'dropzone': true,
    'hidden': true,
    'id': true,
    'lang' : true,
    'spellcheck': true,
    'style': true,
    'tabindex': true,
    'title': true,
    'translate': true,
    'role': true
};

/**
 * Returns a list public methods names available on the component
 * @returns {string[]}
 * @export
 */
InteropComponent.prototype.getPublicMethodNames = function() {};

InteropComponent.prototype.setupMethods = function () {};

/**
 * Function called when an attribute changed in Aura land
 * @param key - { String }
 * @param value - { String }
 */
InteropComponent.prototype.attributeChange = function (key, value) {};

/**
 * re-hydrate each read-only prop that is bound to an attribute (expression)
 */
InteropComponent.prototype.rehydrateBoundReadOnlyProps = function() {};


InteropComponent._classNameCacheMap = {};

/**
 * Creates a hash map for a given className string
 * e.g. `slds-grid slds-col` => { 'slds-grid': true, 'slds-col': true }
 * @param className
 * @returns {Object}
 */
InteropComponent.prototype.getMapFromClassName = function (className) {};

/**
 * Update the class attribute in the custom element, it makes a diff in between the previous class names
 * @param element
 * @param value
 */
InteropComponent.prototype.updateClassAttribute = function (element, value) {};

/**
 * Implement special logic for set HTML Global Attributes
 * @param element {node}
 * @param attrName { string }
 * @param value { any }
 */
InteropComponent.prototype.setGlobalAttribute = function (element, attrName, value) {};



/**
 * Returns the value referenced using property syntax.
 * For example, <code>cmp.get('v.attr')</code> returns the value of the attr aura:attribute.
 *
 * @param {String} key - The data key to look up on the Component.
 * @public
 * @platform
 * @export
 */
InteropComponent.prototype.get = function (key) {};

/**
 * @public
 * @export
 */
InteropComponent.prototype.set = function (key, value) {};

InteropComponent.prototype.attachOnChangeToElement = function (element) {};

/**
 * Render method for Interop components
 * In order to match the lifecycles in Aura and Interop, we need to create a dummy element
 * so the hook for DOM insertion do not get called yet.
 * On the after render we will swap the original version
 *
 * @export
 */
InteropComponent.prototype.render = function () {};

InteropComponent.prototype.setupInteropInstance = function () {};

/*
 * In order to make lifecycle events match we need to swap the element in the afterRender
 * so the insertion hooks get called once the element is on the DOM.
*/
InteropComponent.prototype.swapInteropElement = function (currentElement, newElement) {};

InteropComponent.prototype.updateContainerElement = function (container, currentElement, newElement) {};

/**
 * Invoke the afterRender method
 * @export
 */
InteropComponent.prototype.afterRender = function () {};

/**
 * Invoke the unrender method
 * @export
 */
 InteropComponent.prototype.getElement = function () {};


/**
 * Invoke the unrender method
 * @export
 */
InteropComponent.prototype.unrender = function () {};

/**
 * @public
 * @export
 */
InteropComponent.prototype.destroy = function(){};

/**
 * @public
 * @export
 */
InteropComponent.prototype.find = function(){};

/**
 * @private
 */
InteropComponent.prototype.findInstancesOf = function(){};

/**
 * @private
 */
InteropComponent.prototype.findInstanceOf = function(){};

/**
 *
 * @private
 */
InteropComponent.prototype.getSuperest = function(){};

/**
 * @param {Object} type Applies the type to its definition.
 * @private
 */
InteropComponent.prototype.implementsDirectly = function(){};

/**
 * @protected
 */
InteropComponent.prototype.getRenderer = function() {};

InteropComponent.prototype.getReference = function() {};

/**
 * @private
 */
InteropComponent.prototype.fire = function() {};

/**
 * @public
 * @export
 */
InteropComponent.prototype.getRendering = function(){};

/**
 * @protected
 * @export
 */
InteropComponent.prototype.getSuper = function(){};
/**
 * @public
 * @export
 */
InteropComponent.prototype.getConcreteComponent = function(){};

/**
 * @public
 * @export
 */
InteropComponent.prototype.isConcrete = function() {};

/**
 * @public
 * @export
 */
InteropComponent.prototype.getEventDispatcher = function(){};

/**
 * @public
 * @export
 */
InteropComponent.prototype.getModel = function(){};

/**
 * @public
 * @export
 */
InteropComponent.prototype.getEvent = function() {};

/**
 * @protected
 */
InteropComponent.prototype.getEventByDescriptor = function() {};

/**
 * @public
 * @export
 */
InteropComponent.prototype.toString = function(){};

/**
 * @export
 */
InteropComponent.prototype.getFacets = function() {};

/**
 * @private
 */
InteropComponent.prototype.raiseInvalidInteropApi = function(func, args) {};

Aura.Component.InteropComponent = InteropComponent;
