/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Represents a module containing no default exports.
 *
 * @class InteropModule
 * @constructor
 * @private
 * @export
 */
function InteropModule(config) {}

// Prototype chain (instanceOf)
InteropModule.prototype = Object.create(Aura.Component.InteropComponent.prototype);
InteropModule.prototype.constructor = InteropModule;

/**
 * Returns a list public methods names available on the module
 * @returns {string[]}
 * @export
 */
InteropModule.prototype.getPublicMethodNames = function() {};

/**
 * Adds module keys to instance of InteroptModule
 */
InteropModule.prototype.setupMethods = function () {};

/**
 * No attributes to get on module
 *
 * @param {String} key - The data key to look up on the Component.
 * @public
 * @platform
 * @export
 */
InteropModule.prototype.get = function () {};

/**
 * Cannot set on module
 *
 * @public
 * @export
 */
InteropModule.prototype.set = function () {};

/**
 * Nothing to render on module
 * @export
 */
InteropModule.prototype.render = function () {};

/**
 * Nothing to unrender on module
 * @export
 */
InteropModule.prototype.unrender = function () {};

/**
 * @public
 * @export
 */
InteropModule.prototype.destroy = function() {};


/**
 * @public
 * @export
 */
InteropModule.prototype.toString = function() {};

Aura.Component.InteropModule = InteropModule;
