/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Construct a new IterationComponent.
 *
 * @public
 * @class
 * @constructor
 *
 * @param {Object}
 *            config - component configuration
 * @param {Boolean}
 *            [localCreation] - local creation
 */
function IterationComponent(config, localCreation) {}

IterationComponent.prototype = Object.create(Component.prototype);

// Not returning anything in these since empty functions will not be called by the JS engine as an
IterationComponent.prototype.setupModel = function(){};
IterationComponent.prototype.superRender = function(){};
IterationComponent.prototype.superAfterRender = function(){};
IterationComponent.prototype.superRerender = function(){};
IterationComponent.prototype.superUnrender = function(){};
IterationComponent.prototype.getSuper = function(){};
IterationComponent.prototype.getSuperest = function(){};

IterationComponent.prototype.setupComponentDef = function() {};

IterationComponent.prototype.setupValueProviders = function(customValueProviders) {};

/**
 * Simple type checking. All simple components implement aura:rootComponent and cannot be extended, 
 * so the simple condition here is sufficient unless any of the individual components change.
 */
IterationComponent.prototype.isInstanceOf = function(type) {};

// IterationComponent.ItemValueProvider=function(component){
//     this.get=function(path){
//         throw new Error("DIGGING IN THE RIGHT PLACE");
//     }
// };

IterationComponent.prototype["controller"] = {
    "rangeChange": function(component, evt, helper) {},

    "itemsChange": function(component, evt, helper) {},

    "templateChange":function(component, evt, helper){},

    "init": function(component, evt, helper) {}
};

IterationComponent.prototype["helper"] = {
    createBody: function (component, localCreation) {},
    clearUnrenderedBody: function (component) {},
    updateBody: function (component) {},

    buildBody: function (component, itemHandler, completeHandler) {},

    buildTemplate: function (component, template, item, index, itemVar, indexVar, templateValueProvider, localCreation, forceServer, callback) {},

    getStart: function (cmp) {},

    getEnd: function (cmp) {},

    trackItem: function (component, item, index, components) {}
};

IterationComponent.prototype["provider"] = {
    "provide" : function(component) {}
};

IterationComponent.prototype["renderer"] = {
    "render": function(component){},

    "afterRender": function(component){},

    "rerender": function(component){},

    "unrender" : function(component){}
};

Aura.Component.IterationComponent = IterationComponent;
