    /*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description The Action Definition including the name, descriptor, action type, method, and parameter definitions. An
 *            ActionDef instance is created as part of the ControllerDef initialization.
 *
 * @constructor
 * @param {Object}
 *            config
 * @export
 */
function ActionDef(config) {}

/**
 * Gets the name of this Action. The name is the unique identifier that the component can use to call this Action.
 *
 * @returns {String}
 * @export
 */
ActionDef.prototype.getName = function() {};

/**
 * Gets the Action Descriptor.
 *
 * @returns {String} descriptor of ActionDef
 * @private
 */
ActionDef.prototype.getDescriptor = function() {};

/**
 * Gets the Action type, which can either be "CLIENT" or "SERVER".
 *
 * @returns {String} Possible values are "CLIENT" or "SERVER".
 * @private
 */
ActionDef.prototype.getActionType = function() {};

/**
 * Returns true if the Action type is client-side, or false otherwise.
 *
 * @public
 * @returns {!boolean}
 * @export
 */
ActionDef.prototype.isClientAction = function() {};

/**
 * Returns true if the Action type is server-side, or false otherwise.
 *
 * @public
 * @returns {!boolean}
 * @export
 */
ActionDef.prototype.isServerAction = function() {};

/**
 * Returns true if the action is defined as background (i.e. @BackgroundAction on the java class)
 * @protected
 * @returns {!boolean}
 */
ActionDef.prototype.isBackground = function() {};

/**
 * Returns true if the action is defined as 'force boxcar' (i.e. @CabooseAction on the java class)
 * @protected
 * @returns {!boolean}
 */
ActionDef.prototype.isCaboose = function() {};

/**
 * Returns the action group name if one exists, or undefined if the default should be used
 * @protected
 * @returns {!String}
 */
ActionDef.prototype.getActionGroup = function() {};

/**
 * Returns true if the action is defined to enable public caching (i.e. @PublicCachingEnabled on the java class)
 * @protected
 * @returns {Boolean}
 */
ActionDef.prototype.isPublicCachingEnabled = function() {};

/**
 * Returns the public caching expiration time (in seconds) if the action is defined to enable public caching, 0 otherwise
 * (i.e. the expiration value in @PublicCachingEnabled on the java class)
 * @protected
 * @returns {Number}
 */
ActionDef.prototype.getPublicCachingExpiration = function() {};

/**
 * Returns the action group name if one exists, or undefined if the default should be used
 * @protected
 * @returns {!String}
 */
ActionDef.prototype.getActionGroup = function() {};

/**
 * Returns a new Action instance.
 *
 * @param {Object}
 *            cmp The component associated with the Action.
 * @returns {Action}
 * @private
 */
ActionDef.prototype.newInstance = function(cmp) {};

/**
 * Get a reasonable string representation of the def.
 */
ActionDef.prototype.toString = function() {};

Aura.Controller.ActionDef = ActionDef;
