/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description Creates an Event with name, source component, event definition, event dispatcher, parameters, and sets the fired flag to false.
 * @constructor
 * @param {Object} config
 * @platform
 * @export
 */
Aura.Event.Event = function(config) {};

/**
 * Gets the source component that fired this event.
 *
 * @returns {Object} The source component
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getSource = function() {};

/**
 * Gets the source event that fired this event, if it was fired by an event binding, i.e. {!e.myEvent}.
 *
 * @returns {Object} The source event
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getSourceEvent = function() {};


/**
 * Gets the Event Definition.
 * Returns an EventDef object.
 * @export
 */
Aura.Event.Event.prototype.getDef = function(){};

/**
 * Gets the current phase of this event.
 * Returns undefined if the event has not yet been fired.
 * Possible return values for APPLICATION and COMPONENT events
 * are "capture", "bubble", and "default" once fired.
 * VALUE events return "default" once fired.
 *
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getPhase = function(){};

/**
 * Sets whether the event can bubble or not. This will throw
 * an error if called in the "default" phase.
 * The default is false.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.stopPropagation = function() {};

/**
 * Prevents the default phase execution for this event. This will throw
 * an error if called in the "default" phase.
 * The default is true.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.preventDefault = function() {};

/**
 * Pauses this event such that event handlers will not be processed until
 * Event.resume() is called. The handling process will pause in the current
 * position of the event handler processing sequence. If the event is already
 * paused, calling this does nothing. This will throw an error
 * if called in the "default" phase.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.pause = function() {};

/**
 * Resumes event handling for this event from the same position in the event
 * handler processing sequence from which it was previously paused.
 * If the event is not paused, calling this does nothing. This will throw an error
 * if called in the "default" phase.
 * This API does not define whether or not any remaining event handlers will
 * execute in the current call stack or be deferred and executed in a new call stack,
 * therefore the exact timing behavior is not dependable.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.resume = function() {};

/**
 * Sets the event as a "componentEvent" (won't bubble)
 * This type of event was used historically as a construct to call an action of a child
 * Since the advent of "methods", this type of event communication is discouraged and a "method" is preferred.
 * NOTE: Calling events on a child is discouraged and will be deprecated
 * @export
 */
Aura.Event.Event.prototype.setComponentEvent = function(){};

/**
 * Gets the name of the Event.
 * @returns {String} The event name
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getName = function(){};

/**
 * Gets the type of the Event definition, e.g. 'c:myEvent'.
 * @returns {String} The event definition type
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getType = function(){};

/**
 * Gets the type of the Event, e.g. 'COMPONENT' or 'APPLICATION'.
 * @returns {String} The event type
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getEventType = function(){};

/**
 * Sets parameters for the Event. Does not modify an event that has already been fired.
 * Maps key in config to attributeDefs.
 * @param {Object} config - The parameters for the Event.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.setParams = function(config) {};

/**
 * Sets a parameter for the Event. Does not modify an event that has already been fired.
 * @param {String} key - The name of the parameter.
 * @param {Object} value - The value of the parameter.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.setParam = function(key, value) {};

/**
 * Gets an Event parameter.
 * @param {String} name The name of the Event. For example, <code>event.getParam("button")</code> returns the value of the pressed mouse button (0, 1, or 2).
 * @returns {Object} The parameter value
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getParam = function(name){};

/**
 * Gets all the Event parameters.
 * @returns {Object} The collection of parameters
 * @platform
 * @export
 */
Aura.Event.Event.prototype.getParams = function(){};


/**
 * Convenience function to determine which kind of event execution should be used
 * for this event. This is NOT PUBLIC! It's NOT an API and can change at any point.
 * @return {String} One of: "VALUE", "COMPONENT", "LEGACY_COMPONENT", "APPLICATION"
 * @private
 */
Aura.Event.Event.prototype.getEventExecutionType = function () {};

/**
 * Executes the event handlers for this event.
 * @private
 */
Aura.Event.Event.prototype.executeHandlers = function() {};

/**
 * Executes the handlers returned by handlerIterator.
 * @param {Iterator} handlerIterator
 * @private
 */
Aura.Event.Event.prototype.executeHandlerIterator = function(handlerIterator) {};

/**
 * Returns an iterator over the handlers for this event.
 * @private
 */
Aura.Event.Event.prototype.getHandlerIterator = function() {};

/**
 * Fires the Event. Checks if the Event has already been fired before firing.
 * Maps the component handlers to the event dispatcher.
 * @param {Object} params - Optional. A set of parameters for the Event. Any previous parameters of the same name will be overwritten.
 * @platform
 * @export
 */
Aura.Event.Event.prototype.fire = function(params) {};
