/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description We use an `Aura.Event.Event`-like instance instead of Aura.Event.Event
 *              itself because we need to pass `event.target` and `event.currentTarget`
 *              in some cases for backwards-compatibility, and locker does not allow
 *              expandos on instances of `Aura.Event.Event`.
 *
 * @constructor InteropEvent
 *
 * @param {Component} component - Required. The return value of `event.getSource()`.
 * @param {Object} config
 * @param {boolean} config.isEvent - The native event associated with this
 * @param {Object} config.params - If config.isEvent: The native event associated with this AuraEvent,
 *                                 otherwise the params of the AuraEvent.
 * @param {Object} config.exposeNativeAPI - If config.isEvent then it exposes the native event API
 *                                          for backwards-compatibility. Should not be used for new components.
 *
 * @private
 * @export
 */
Aura.Event.InteropEvent = function (component, config) {};

/**
 * Needed to unwrap Proxies that might come as part of the event.details and wont work on Compat mode.
 * @private
 * @param eventDetails
 */
Aura.Event.InteropEvent.prototype.buildEventParams = function (eventDetails) {};

/**
 * @private
 * @param auraEvent
 * @param nativeEvent
 */
Aura.Event.InteropEvent.prototype.exposeNativeEventAPI = function (auraEvent, nativeEvent) {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.fire = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.getName = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.getParam = function (name) {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.getParams = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.getPhase = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.getSource = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.pause = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.preventDefault = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.resume = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.setParam = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.setParams = function () {};

/**
 * @export
 */
Aura.Event.InteropEvent.prototype.stopPropagation = function () {};

/**
 * @private
 */
Aura.Event.InteropEvent.prototype.raiseInvalidInteropApi = function(func, args) {};