/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * A registry of library includes.
 * Includes are similar to RequireJS's modules. The difference is their comsumption:
 * in Aura, those are exported as items on library s instead of inside a require callback.
 * We delay the creation of the instance of a library include untill it's requested.
 * @constructor
 */
function LibraryIncludeRegistry() {}

/**
 * Detects if the library include exists without actually defining it.
 * @param {String} descriptor The qualified name of the library include in the form markup://namespace:include
 */
LibraryIncludeRegistry.prototype.hasLibraryInclude = function(descriptor) {};

/**
 * The function that handles definitions of library includes. This works like a simplfied version of
 * RequireJS's "define" function with a fixed set of arguments.
 * @param {String} descriptor name of the include.
 * @param {Array} dependencies The list of dependencies (other includes).
 * @param {Function} exporter A function that when executed will return the include object.
 */
 LibraryIncludeRegistry.prototype.addLibraryInclude = function(descriptor, dependencies, exporter) {};

/**
 * The function that handles definitions of library includes.
 * @param {String} descriptor name of the include.
 * @param {Function} exporter A function that when executed will return the include object.
 */
LibraryIncludeRegistry.prototype.addLibraryExporter = function(descriptor, exporter) {};


/**
 * Get or build the instance for the specified library include. his works like a simplfied version of
 * RequireJS's "require" function, without a callback.
 * @param {String} descriptor in the form markup://namespace:include.
 * @returns Either the instance of the include you are requesting, or undefined if not found.
 * @export
 */
LibraryIncludeRegistry.prototype.getLibraryInclude = function(descriptor) {};

LibraryIncludeRegistry.prototype.hydrateLibrary = function(descriptor, exporter) {};

/**
 * Try to build an instance for the specified library include.
 * @param {Array} dependencies The list of descriptors markup://namespace:include.
 * @returns {Array} the list of instances or undefined.
 */
LibraryIncludeRegistry.prototype.buildLibraryInclude = function(descriptor) {};

/**
 * Reset the queue.
 */
LibraryIncludeRegistry.prototype.clearDependencyQueue = function() {};

/**
 * Add a dependcy to the processing queue.
 * @param {String} descriptor The qualified name of the library include in the form markup://namespace:include
 */
LibraryIncludeRegistry.prototype.enqueueDependency = function(descriptor) {};

/**
 * Remove a dependcy from the processing queue.
 * @param {String} descriptor The qualified name of the library include in the form markup://namespace:include
 */
LibraryIncludeRegistry.prototype.dequeueDependency = function(descriptor) {};

/**
 * Clone the array of dependencies and reset the changed flag.
 * @return {Array} The cloned array of dependecies.
 */
LibraryIncludeRegistry.prototype.cloneDependencyQueue = function() {};

/**
 * Return the modified status of the dependency queue.
 * @return {Boolean} True if the dependency queue has changed.
 */
LibraryIncludeRegistry.prototype.hasDependencyQueue = function() {};

/**
 * Return the modified status of the dependency queue.
 * @return {Boolean} True if the dependency queue has changed.
 */
LibraryIncludeRegistry.prototype.hasDependencyQueueChanged = function() {};

Aura.Library.LibraryIncludeRegistry = LibraryIncludeRegistry;
