/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * A registry to combine library definitions and library instance.
 * @constructor
 */
function LibraryRegistry() {}

/**
 * Init library definitions.
 * @param {Array} Library definitions.
 * @export
 */
LibraryRegistry.prototype.initLibraries = function(libraries) {};

/**
 * Detects if the library exists without actually defining it.
 * @param {String} descriptor The qualified name of the library in the form markup://namespace:library
 */
LibraryRegistry.prototype.hasLibrary = function(descriptor) {};

/**
 * Register a library defintion. A library is a collection of includes.
 * @param {String} descriptor The qualified name of the library in the form markup://namespace:library
 * @param {Object} includes Pairs of library export and include definitions.
 */
LibraryRegistry.prototype.addLibrary = function(descriptor, includes) {};

/**
 * Returns a library from the registry. If the library is missing,
 * build the library before returning it.
 * @param {String} descriptor The qualified name of the library in the form markup://namespace:library
 * @returns {Object} library from registry.
 */
LibraryRegistry.prototype.getLibrary = function(descriptor) {};

/**
 * Returns a library instance which is a map of library include instances.
 * @param {Array} includes The map of incudes, export and descriptor pairs.
 * @returns {Object} new library.
 */
 LibraryRegistry.prototype.buildLibrary = function(includes) {};

Aura.Library.LibraryRegistry = LibraryRegistry;