/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * @description The Aura Metrics Service, accessible using <code>$A.metricsService</code>.
 * @constructor
 * @export
**/
Aura.Services.MetricsService = function MetricsService() {};

// Version
Aura.Services.MetricsService.VERSION  = '2.2.0';

Aura.Services.MetricsService.PERFTIME     = !!(window.performance && window.performance.now);
Aura.Services.MetricsService.TIMER        = Aura.Services.MetricsService.PERFTIME ? function () {} : Date.now.bind(Date);
Aura.Services.MetricsService.START        = 'start';
Aura.Services.MetricsService.END          = 'end';
Aura.Services.MetricsService.STAMP        = 'stamp';
Aura.Services.MetricsService.DEFAULT      = 'default';
Aura.Services.MetricsService.MAXTIME      = 30000; // Max time for a transaction to finish
Aura.Services.MetricsService.CUSTOM_MARKS = 'custom';

/**
 * Initialize function
 *@private
**/
Aura.Services.MetricsService.prototype.initialize = function () {};

/**
 * Instrument a particular method (function) of an object, useful for AOP
 * @param {Object} instance Object that holds the method to hook
 * @param {string} method Method name
 * @param {string} ns Namespace
 * @param {boolean} async
 * @param {function(Object)} before
 * @param {function} after
 * @param {function} override
 * @export
**/
Aura.Services.MetricsService.prototype.instrument = function (instance, method, ns, async, before, after, override) {};

/**
 * UnInstrument a particular method (function) of an object, useful for AOP
 * @param {Object} instance Object that holds the method to hook
 * @param {string} method Method name
 * @export
**/
Aura.Services.MetricsService.prototype.unInstrument = function (instance, method) {};

/**
 * Initialize registered plugins
 * @private
**/
Aura.Services.MetricsService.prototype.initializePlugins = function () {};

/**
 * Initialize a give plugin
 * @param {string} pluginName Plugin name
 * @param {Object|function} Construtor or Objecct for the plugin
 * @private
**/
Aura.Services.MetricsService.prototype.initializePlugin = function (pluginName, PluginContructor) {};

/**
 * Internal method called once the application is ready
 * @private
**/
Aura.Services.MetricsService.prototype.applicationReady = function () {};

Aura.Services.MetricsService.prototype.emitBootstrapTransaction = function () {};

/**
 * Create a transaction based on a given configuration
 * @param {string} ns Namespace of the transaction
 * @param {string} name Name of the transaction
 * @param {Object} transaction Transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.syntheticTransactionStart = function (ns, name, config) {};


/**
 * Add a callback everytime a transaction ends.
 * @param {function} callback Function to execute for every transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.onTransactionEnd = function (callback) {};

/**
 * Unbind a callback everytime a transaction ends.
 * @param {function} callback Function to detach for every transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.detachOnTransactionEnd = function (callback) {};

/**
 * Add a callback everytime a transaction ends.
 * @param {function} callback Function to execute for every transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.onTransactionsKilled = function (callback) {};

/**
 * Unbind a callback everytime a transaction ends.
 * @param {function} callback Function to detach for every transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.detachOnKilledTransactions = function (callback) {};

/**
 * Unbind a callback for a give action
 * @param {function} callback Function to detach for every transaction
 * @param {name} callback Function to detach for every transaction
**/
Aura.Services.MetricsService.prototype.detachHandlerOfType = function (callback, name) {};



/**
 * Check if we are in a transcation already
 * @return {boolean} Wether we are in a transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.inTransaction = function (ignorePageTransactions) {};



/**
 * Create a transaction
 * @param {string} ns Namespace of the transaction
 * @param {string} id Id of the transaction
 * @param {Object} config Configuration and context for the transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.transaction = function (ns, name, config) {};


/**
 * Update a transaction
 * @param {string} ns Namespace of the transaction
 * @param {string} id Id of the transaction
 * @param {Object} config Configuration and context for the transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.transactionUpdate = function (ns, name, config) {};

/**
 * Start a transaction
 * @param {string} ns Namespace of the transaction
 * @param {string} id Id of the transaction
 * @param {Object} config Configuration and context for the transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.transactionStart = function (ns, name, config) {};

/**
 * Finish a transaction
 * @param {string} ns Namespace of the transaction
 * @param {string} id Id of the transaction
 * @param {Object} config Configuration and context for the transaction
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.transactionEnd = function (ns, name, config, postProcess) {};


/**
 * Clear all saved transactions
 * @public
 * @export
**/

Aura.Services.MetricsService.prototype.clearTransactions = function () {};

/**
 * Internal method to call globalHandlers attached
 * @private
**/
Aura.Services.MetricsService.prototype.callHandlers = function (type, t) {};

/**
 * Tries to kill transaction than are been in the queue for a long period of time
 * defined via static param on metricsService
 * @private
**/
Aura.Services.MetricsService.prototype.killLongRunningTransactions = function () {};


/**
 * Get the current page transaction
 * has pageTransaction config flag
 * @export
**/
Aura.Services.MetricsService.prototype.getCurrentPageTransaction = function () {};

/**
 * Merge the config of the current page transaction
 * @export
**/
Aura.Services.MetricsService.prototype.updateCurrentPageTransaction = function (config) {};

/**
 * Returns a clone of the marks currently available on the collector
 * @export
**/
Aura.Services.MetricsService.prototype.getCurrentMarks = function () {};

/**
 * Returns the metricsService version
 * @export
**/
Aura.Services.MetricsService.prototype.getVersion = function (includePlugins) {};



/**
 * Default post processing for marks (only enabled in non production environments)
 * @private
**/
Aura.Services.MetricsService.prototype.defaultPostProcessing = function (customMarks) {};

//#if {"excludeModes" : ["PRODUCTION"]}

/**
 * Get all internal stored transactions
 * @export
**/
Aura.Services.MetricsService.prototype.getTransactions = function () {};

/**
 * Get a internal stored transaction by id
 * @param {?string} ns Namespace of the transaction
 * @param {string} id Id of the transaction
 * @export
**/
Aura.Services.MetricsService.prototype.getTransaction = function (ns, id) {};

/**
 * Set the internal storage of transactions
 * @export
**/
Aura.Services.MetricsService.prototype.setClearCompletedTransactions = function (value) {};

//#end

/**
 * Creates a transaction
 * @param {string} ns Namespace of the mark
 * @param {string} name of the mark
 * @param {Object} config Config object
 * @private
**/
Aura.Services.MetricsService.prototype.createTransaction = function (ns, name, config) {};


/**
 * Creates a mark for a give namespace and key action
 * @param {string} ns Namespace of the mark
 * @param {string} name of the mark
 * @param {Object} context Context Object
 * @export
 * @public
**/
Aura.Services.MetricsService.prototype.mark = function (ns, name, context) {};


/**
 * Creates a start mark for a give namespace and key action
 * @param {string} ns Namespace of the mark
 * @param {string} name of the mark
 * @param {Object} context Context Object
 * @export
 * @public
**/
Aura.Services.MetricsService.prototype.markStart = function (ns, name, context) {};


/**
 * Creates a end mark for a give namespace and key action
 * @param {string} ns Namespace of the mark
 * @param {string} name of the mark
 * @param {Object} context Context Object
 * @export
 * @public
**/
Aura.Services.MetricsService.prototype.markEnd = function (ns, name, context) {};


/**
 * Creates a mark node
 * @param {string} ns Namespace of the mark
 * @param {string} name of the mark
 * @param {string} eventType Type of the mark
 * @param {Object} options Options
 * @private
**/
Aura.Services.MetricsService.prototype.createMarkNode = function (ns, name, eventType, options) {};

/**
 * Clear Marks
 * @param {?string=} ns Namespace of the marks to clean
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.clearMarks = function (ns) {};

/**
 * Get the page firstByte timestamp from either performance API or a mark in the page
 * @private
**/
Aura.Services.MetricsService.prototype.getPageStartTime = function () {};

/**
 * Generates a bootstrap mark
 * @param {string} mark Key of the mark
 * @param {?} value Value of the mark
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.time = function () {};

/**
 * Returns if the resolution is microseconds (using the performance API if supported)
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.microsecondsResolution = function () {};


/**
 * Disable all plugins
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.disablePlugins = function () {};

/**
 * Diable a plugin by name
 * @param {string} name Name of the plugin
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.disablePlugin = function (name) {};

/**
 * Enable all plugins
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.enablePlugins = function () {};

/**
 * Enable plugin by name
 * @param {string} name Name of the plugin
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.enablePlugin = function (name) {};

/**
 * Register a plugin for metricsServices
 * @param {Object} pluginConfig A plugin object
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.registerPlugin = function (pluginConfig) {};


/**
 * Register a beacon (transport layer) on which to send the finishes transactions
 * @param {Object} beacon Beacon object that hold a "sendData" method
 * @public
 * @export
**/
Aura.Services.MetricsService.prototype.registerBeacon = function (beacon) {};
/**
 * Summarizes perf timing request info
 * @param {PerformanceResourceTiming} r
 * @export
*/
Aura.Services.MetricsService.prototype.summarizeResourcePerfInfo = function (r) {};

/**
 * Prevent the framework from logging the bootstrap transaction
 * This will be handled by the calling app at some point in the future
 * @export
*/
Aura.Services.MetricsService.prototype.skipBootstrapLogging = function () {};

/**
 * Returns a JSON Object which contains the bootstrap metrics of the framework and the application
 * @public
 * @return {Object}
 * @export
**/
Aura.Services.MetricsService.prototype.getBootstrapMetrics = function () {};
