/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * DomHandlersPlugin
 * =================
 * This plugin hooks into the definition of aura:html component.
 * In particular the following methods:
 * dispatchAction(): Tracks all interactions (click, mouseovers, any DOM handlers) 
 * handled by any active component
 *
 * @description DomHandlersPlugin
 * @constructor
 * @export
 */
var DomHandlersPlugin = function DomHandlersPlugin(config) {};

DomHandlersPlugin.NAME = "domHandlers";
DomHandlersPlugin.DEFAULT_INTERACTION_TYPE = "user";
DomHandlersPlugin.WHITELISTEVENTS = { 
    "click"  : true,
    "change" : true
};

/** @export */
DomHandlersPlugin.prototype.initialize = function (metricsService) {};

/** @export */
DomHandlersPlugin.prototype.enable = function () {};

/** @export */
DomHandlersPlugin.prototype.disable = function () {};

/*
// We might need this method somewhere in the future

DomHandlersPlugin.prototype.stringifyLocator = function (locator) {
    var ordered = { 
        target  : locator.target, 
        scope   : locator.scope, 
        context : locator.context && Object.keys(locator.context).sort().reduce(function (r, k) { r[k] = locator.context[k]; return r; }, {}) 
    };

    if (!locator.context) {
        delete locator.context;
    }
    
    return JSON.stringify(ordered);
};
*/

/**
 * Logs LightningInteraction transactions if the html event handled is in the whitelist
 * 
 * @param action - This is unused
 * @param event - The DOM Event
 * @param root - The aura:html component that is handling the event
 */
DomHandlersPlugin.prototype.dispatchActionHook = function (action, event, root) {};

//#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
DomHandlersPlugin.prototype.logUnInstrumentedClick = function (parent, root) {};
//#end

DomHandlersPlugin.prototype.bindToMetricsService = function (jsObject, method) {};

DomHandlersPlugin.prototype.bindToHelper = function (descriptor, helperMethod) {};

DomHandlersPlugin.prototype.bindToLib = function (lib, jsFile) {};

DomHandlersPlugin.prototype.bind = function () {};

DomHandlersPlugin.prototype.instrumentCallback = function (/*original*/) {};

//#if {"excludeModes" : ["PRODUCTION"]}
/** @export */
DomHandlersPlugin.prototype.postProcess = function (transportMarks) {};
//#end

DomHandlersPlugin.prototype.unbind = function () {};

$A.metricsService.registerPlugin({
    "name"   : DomHandlersPlugin.NAME,
    "plugin" : DomHandlersPlugin
});
