/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description A base class to enable JSON manipulation
 * @constructor
 */
function Json() {}

Json.ApplicationKey={
    //#json.applicationKeys
};

/**
 * Decode a JSON string into an object, optionally using ref support to resolve
 * duplicate object references.
 *
 * IMPORTANT: this function should NEVER be exported, as the eval()
 * implementation provides a public API for executing arbitrary code inside our
 * domain security context. If we decide we need to supply a JSON utility
 * function to consumers in the future, it should use the new window.JSON
 * support provided by newer browsers.
 *
 * @param {String} json
 * @param {Object} refSupport
 */
Json.prototype.decode = function(json, refSupport) {};

/**
 * After the JSON data is decoded by Browser native JSON object, the resulted
 * object (or sub-object) property's value could be a function/array/object/ and
 * so on in string format. We need to convert them to the desired type.
 *
 * @param {String} value The string to be decoded
 * @returns {Function|Array|Object} The converted value
 */
Json.prototype.decodeString = function(value) {};

/**
 * Convert a serialized state blob, which is returned from a server action, which
 * contains internal serId and serRefId markers, into a new data structure that
 * has internal JavaScript pointers to the same object. Component definitions are
 * extracted and placed into <code>context.componentDefs</code>.
 *
 * @param {Object} obj The object to resolve. It is modified in-place.
 */
Json.prototype.resolveRefsObject = function(obj) {};

/**
 * Convert a serialized state blob, with its internal serId and serRefId
 * markers, into a new data structure that has internal JavaScript pointers
 * to the same object. Component definitions are extracted and placed at
 * the front of the array.
 *
 * @param {Array} arr The array to resolve. It is modified in-place.
 */
Json.prototype.resolveRefsArray = function(arr) {};

Json.prototype._resolveRefs = function(config, cache, parent, property, collector) {};

Json.prototype.stringifyReplacer = function(key, value) {};

/**
 * Encodes an object into a JSON representation.
 *
 * @param {Object} obj The object to pass in the encoder.
 * @param {Object} replacer Optional function which passes key and value bound to the
 *            object, and returns a stringified value.
 * @param {String} whiteSpace Adds spaces or tabs to the resulting string. E.g. '\t'
 *            for tab
 */
Json.prototype.encode = function(obj, replacer, whiteSpace) {] = Json.prototype.encode;

