/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description Label Provider. Performs server action to retrieve label values
 * @constructor
 */
function LabelValueProvider(values) {}

/**
 * Performs LabelController.getLabel action to get specified section and name.
 * Sets up label queue so that server action for the same label is only requested once
 *
 * @param {String} section - label section
 * @param {String} name - label section
 * @param {Function} [callback] - callback
 * @return {String}
 * @private
 */
LabelValueProvider.prototype.requestServerLabel = function(section, name, callback) {};

/**
 * Gets queue for specified label
 *
 * @param {String} section - label section
 * @param {String} name - label name
 * @return {LabelQueue} queue for given label
 */
LabelValueProvider.prototype.getQueue = function(section, name) {};

/**
 * Removes label queue
 * @param {String} section - label section
 * @param {String} name - label name
 */
LabelValueProvider.prototype.removeQueue = function(section, name) {};

/**
 * Gets label key in queue
 * @param {String} section - label section
 * @param {String} name - label name
 */
LabelValueProvider.prototype.getQueueKey = function(section, name) {};

/**
 * returns $Label values
 */
LabelValueProvider.prototype.getValues = function() {};

/**
 * Merges $Label values
 */
LabelValueProvider.prototype.merge = function(values) {};

/**
 * Loops through existing values to find value. If no value found, send request to server
 *
 * @param {String} expression - expression
 * @param {Component} [component] - component
 * @param {Function} [callback] - callback
 * @return {String}
 */
LabelValueProvider.prototype.get = function(expression, callback) {};

Aura.Provider.LabelValueProvider = LabelValueProvider;