
/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @description
 * Util methods provide utility functions for browsers in addition to
 * functions for retrieving, manipulating, or checking DOM elements.
 * @constructor
 * @platform
 * @export
 */
Aura.Utils.Util = function Util() {};

/**
 * Browser check for all supported versions of Internet Explorer, does the validation using the userAgent.
 *
 * @returns {Boolean} true if Internet Explorer detected
 * @export
 */
Aura.Utils.Util.prototype.isIE = (navigator.userAgent.indexOf("MSIE") !== -1) || (navigator.userAgent.indexOf("Trident/") !== -1);

Promise.prototype.then = Promise.prototype['then'];

/**
 * Browser check for localStorage enablement.
 * @returns {Boolean} true if LocalStorage is available
 * @private
 */
Aura.Utils.Util.prototype.isLocalStorageEnabled = function () {};

/**
 * Browser check for sessionStorage enablement.
 * @returns {Boolean} true if SessionStorage is available
 * @private
 */
Aura.Utils.Util.prototype.isSessionStorageEnabled = function () {};



/**
 * Sets a cookie.
 * @param {String} key The name of the cookie.
 * @param {String} value The value of the cookie.
 * @param {Number} duration The duration of the cookie (milliseconds)
 * @private
 */
Aura.Utils.Util.prototype.setCookie = function(key, value, duration) {};

/**
 * Gets a cookie.
 * @param {String} key The name of the cookie.
 * @return {String} the value of the cookie if it exists, undefined otherwise.
 * @export
 */
Aura.Utils.Util.prototype.getCookie = function(key) {};

/**
 * Clears a cookie.
 * @param {String} key The name of the cookie.
 */
Aura.Utils.Util.prototype.clearCookie = function(key) {};

/**
 * Whether IOS7 UIWebView
 * @returns {boolean} true if IOS UIWebView
 * @export
 */
Aura.Utils.Util.prototype.isIOSWebView = function() {};

/**
 * evals code globally, without enclosing the current scope
 *
 * @private
 */
Aura.Utils.Util.prototype.globalEval = function(src, sourceURL) {};

/**
 * Checks whether the specified object is an array.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is an array, or false otherwise.
 * @function
 * @platform
 * @export
 */
Aura.Utils.Util.prototype.isArray = typeof Array.isArray === "function" ? Array.isArray : function(arg) {};

/**
 * Checks whether the specified object is a valid object.
 * A valid object: Is not a DOM element, is not a native browser class (XMLHttpRequest)
 * is not falsey, and is not an array, error, function string or number.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is a valid object, or false otherwise.
 * @function
 * @platform
 * @export
 */
Aura.Utils.Util.prototype.isObject = function(obj){};

/**
 * Checks whether the specified object is a plain object or literal.
 * A plain object is created using "{}" or "new Object()".
 *
 * @param {Object} o The object to check for.
 * @returns {Boolean} True if the object is a plain object, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isPlainObject = function(o){};

/**
 * Checks whether the specified object is a valid error.
 * A valid error: Is not a DOM element, native browser class (XMLHttpRequest), falsey,
 * array, function string or number.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is a valid error, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isError = function(obj){};

/**
 * Checks whether the specified object is a valid function.
 * A valid function: Is not a DOM element, native browser class (XMLHttpRequest), falsey,
 * array, error, or number.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is a valid function, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isFunction = function(obj){};

/**
 * Checks if the object is of type string.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is of type string, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isString = function(obj){};

/**
 * Checks if the object is of type number.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is of type number, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isNumber = function(obj){};

/**
 * Checks if the object is a finite number (not NaN or Infinity or -Infinity)
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is a finite number, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isFiniteNumber = function(obj){};

/**
 * Checks if the object is of type boolean.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is of type boolean, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isBoolean = function(obj){};

/**
 * Checks if the object is undefined.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is undefined, or false otherwise.
 * @export
 * @platform
 */
Aura.Utils.Util.prototype.isUndefined = function(obj){};

/**
 * Checks if the object is undefined or null.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is undefined or null, or return false otherwise.
 * @export
 * @platform
 */
Aura.Utils.Util.prototype.isUndefinedOrNull = function(obj){};

/**
 * Checks if the object is empty.
 * An empty object's value is undefined, null, an empty array, or empty string. An object with no native
 * properties is not considered empty.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object is empty, or false otherwise.
 * @platform
 * @export
 */
Aura.Utils.Util.prototype.isEmpty = function(obj){};

/**
 * Coerces truthy and falsy values into native booleans
 *
 * @param {Object} val The object to check.
 * @returns {Boolean} True if the object is truthy, or false otherwise.
 * @platform
 * @function
 * @export
 */
Aura.Utils.Util.prototype.getBooleanValue = function (val) {};

/**
 * Creates and returns an HTML element of the specified tag name and map of attributes.
 *
 * @param {String} tagName Tag name of the html element to create (e.g. 'a', 'img', 'div').
 * @param {Map} attributes A map of attributes that the element will have
 *   (e.g. {src: 'foo.img', alt: 'Some text'}
 * @returns (HTMLElement) the newly created element
 * @export
 */
Aura.Utils.Util.prototype.createHtmlElement = function (tagName, attributes) {};

/**
 * Removes all children of a node, effectively clearing its body.
 *
 * @param {HTMLElement} node The node to be cleared.
 * @export
 */
Aura.Utils.Util.prototype.clearNode = function (node) {};

/**
 * Sanitize
 * Public method providing core sanitation functionality
 *
 * @param {String} dirty string
 * @param {Object} configuration object
 * @return {String} sanitized string
 * @export
*/

Aura.Utils.Util.prototype.sanitizeDOM = function (dirty, config) {};

// -- Sanitize Facade (secure-filters) -----------------------------------------------

/**
 * Encodes values for safe embedding in HTML tags and attributes.
 *
 * @param {*} val will be converted to a String prior to encoding
 * @return {string} the encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeHtml = Aura.Utils.SecureFilters.html;

/**
 * Encodes values for safe embedding in JavaScript string contexts.
 *
 * @param {*} val will be converted to a String prior to encoding
 * @return {string} the encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeJs = Aura.Utils.SecureFilters.js;


/**
 * Encodes values embedded in HTML scripting attributes.
 *
 * @param {*} val will be converted to a String prior to encoding
 * @return {string} the encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeJsAttr = Aura.Utils.SecureFilters.jsAttr;

/**
 * Percent-encodes unsafe characters in URIs.
 *
 * @param {*} val will be converted to a String prior to encoding
 * @return {string} the percent-encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeUri = Aura.Utils.SecureFilters.uri;


/**
 * Encodes an object as JSON, but with unsafe characters in string literals
 * backslash-escaped.
 *
 * @param {any} val
 * @return {string} the JSON and backslash-encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeJsObj = Aura.Utils.SecureFilters.jsObj;


/**
 * Encodes values for safe embedding in CSS context.
 *
 * @param {any} val
 * @return {string} the backslash-encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeCSS = Aura.Utils.SecureFilters.css;


/**
 * Encodes values for safe embedding in HTML style attribute context.
 *
 * @param {any} val
 * @return {string} the entity and backslash-encoded string
 * @export
*/
Aura.Utils.Util.prototype.sanitizeStyle = Aura.Utils.SecureFilters.style;


// -----------------------------------------------------------------------------------

/**
 * Gets a DOM element by its id without any leading characters (e.g. #) unless the ID contains them.
 *
 * @param {String} id The corresponding id of the DOM element.
 * @returns {Object} The element with the matching id, or null if none is found.
 * @export
 */
Aura.Utils.Util.prototype.getElement = function(id){};

/**
 * Gets a copy of an object. In the case of an Array or Object, returns a shallow copy. In the case of a literal,
 * returns the literal value.
 *
 * @param {Object} value The value for which to return a comparable copy.
 * @returns {Object} The comparable copy of the value supplied.
 */
Aura.Utils.Util.prototype.copy = function(value){};

/**
 * Compares value equality of two variables. Returns true if primitive values match,
 * or if Object or Array members contain the same values. Checks Objects and Arrays recursively.
 * Fails fast. Note that objects stop comparing after satisfying 'likeness' against 'expected' -- i.e.,
 * 'actual' is allowed to exhibit members that 'expected' does not, as long as all members of 'expected' are matched.
 *
 * @param {Object} expected The baseline value to use in the comparison against 'actual'.
 * @param {Object} actual The value to compare against 'expected'.
 * @returns {Boolean} Returns true if the values match, and false if they do not.
 */
Aura.Utils.Util.prototype.equals = function(expected,actual){};

/**
 * Compares values. In the case of an Array or Object, compares first level references only.
 * In the case of a literal, directly compares value and type equality.
 * USE IN TESTS ONLY. HAS NON-TRIVIAL PERFORMANCE IMPLICATIONS.
 *
 * @param {Object} expected The source value to compare.
 * @param {Object} actual The target value to compare.
 * @returns {Object} The result of the comparison, with reasons.
 */
Aura.Utils.Util.prototype.compareValues = function(expected, actual){};


/**
 * Checks whether the component has the specified CSS class.
 *
 * @example
 * //find a component with aura:id="myCmp" in markup
 * var myCmp = component.find("myCmp");
 * $A.util.hasClass(myCmp, "myClass");
 *
 * @param {Object} element The component to check.
 * @param {String} className The CSS class name to check for.
 * @returns {Boolean} True if the specified class is found for the component, or false otherwise.
 * @export
 * @platform
 *
 */
Aura.Utils.Util.prototype.hasClass = function(element, className){};

/**
 * Adds a CSS class to a component.
 *
 * @example
 * //find a component with aura:id="myCmp" in markup
 * var myCmp = component.find("myCmp");
 * $A.util.addClass(myCmp, "myClass");
 *
 * @param {Object} element The component to apply the class on.
 * @param {String} newClass The CSS class to be applied.
 * @export
 * @platform
 */
Aura.Utils.Util.prototype.addClass = function(element, newClass){};

/**
 * Removes a CSS class from a component.
 *
 * @example
 * //find a component with aura:id="myCmp" in markup
 * var myCmp = component.find("myCmp");
 * $A.util.removeClass(myCmp, "myClass");
 *
 * @param {Object} element The component to remove the class from.
 * @param {String} newClass The CSS class to be removed from the element.
 * @export
 * @platform
 *
 */
Aura.Utils.Util.prototype.removeClass = function(element, newClass){};

/**
 * Toggles (adds or removes) a CSS class from a component.
 *
 * @example
 * //find a component with aura:id="toggleMe" in markup
 * var toggleText = component.find("toggleMe");
 * $A.util.toggleClass(toggleText, "toggle");
 *
 * @param {Object} element The component to add or remove the class from.
 * @param {String} className The CSS class to be added or removed.
 * @export
 * @platform
 *
 */
Aura.Utils.Util.prototype.toggleClass = function(element, className, condition){};

/**
 * Swaps an element's class by removing the selected class and adding another in its place.
 *
 * @param {Object} element The element to be processed.
 * @param {String} oldClass The class to remove from the element.
 * @param {String} newClass The class to add to the element.
 * @export
 */
Aura.Utils.Util.prototype.swapClass = function(element, oldClass, newClass){};

/**
 * @private
 */
Aura.Utils.Util.prototype.setClass=function(element,newClass,remove){};

/**
 * @private
 */
Aura.Utils.Util.prototype.buildClass=function(oldClass, newClass, remove){};

/**
 * Builds the appropriate css class name for a flavor.
 *
 * @param {Object} cmp The DefDescriptor of the component being flavored.
 * @returns {String} flavor The flavor name.
 * @export
 */
Aura.Utils.Util.prototype.buildFlavorClass = function(cmp, flavor) {};

/**
 * Generates dom nodes from string markup
 *
 * @param {String} markup The markup from which to generate dom nodes
 * @returns {Array} An array of the elements that were generated.
 * @export
 */
Aura.Utils.Util.prototype.createElementsFromMarkup=function(markup){};

/**
 * Inserts element(s) as the first child of the parent element.
 *
 * @param {Object} newE1 The new element to insert.
 * @param {Object} referenceE1 The reference element
 * @returns {Object} The element that was inserted.
 * @export
 */
 Aura.Utils.Util.prototype.insertFirst = function(newEl, referenceEl){};

/**
 * Inserts a new element, newEl, directly before the reference element, referenceEl.
 * If the reference element is a parent node, insert the new element directly before the parent node.
 *
 * @param {Object} newE1 The new element to insert.
 * @param {Object} referenceE1 The reference element
 * @returns {Object} The element that was inserted.
 * @export
 */
Aura.Utils.Util.prototype.insertBefore = function(newEl, referenceEl) {};

/**
 * Inserts a new element, newEl, directly after the reference element, referenceEl.
 * If the reference element is a parent node, insert the new element directly after the parent node.
 *
 * @param {Object} newE1 The new element to insert.
 * @param {Object} referenceE1 The reference element
 * @returns {Object} The element that was inserted.
 * @export
 */
Aura.Utils.Util.prototype.insertAfter = function(newEl, referenceEl) {};

/**
 * Adds a new element to the end of the reference element. Does not work if the canHaveChildren property on the reference element is false.
 *
 * @param {Object} newE1 The element to append as a child of the reference element.
 * @param {Object} referenceE1 The existing element
 * @returns {Object} The new element that was added
 */
Aura.Utils.Util.prototype.appendChild = function(newEl, referenceEl) {};

/**
 * Removes the specified element from the DOM.
 *
 * Use this method with caution. Since we hijack the normal delete
 * functionality, we need to be careful of odd event processing. Specifically
 * we end up sending off some events that would not otherwise be sent.
 *
 * Also note that we currently remove nodes children first, which means we
 * deconstruct our tree from the bottom up. If we reverse this, we might be
 * able to add optimizations.
 *
 * @param {Object} element The element to be removed.
 * @export
 */
Aura.Utils.Util.prototype.removeElement = function(element) {};

/**
 * Manipulate the properties of the querystring portion of a url.
 * @param {String} url Any url to manipulate, if it doesn't have a question mark in it. Any hash remains not affected.
 * @param {Object} params Map of key->value's to set in the url. Set key to null to remove it from the url.
 * @param {Boolean} encoded True if params are alredy encoded and prevent re-encoding.
 * @export
 */
Aura.Utils.Util.prototype.generateUrl = function(url, params, encoded) {};

/**
 * Trims a string by removing newlines, spaces, and tabs from the beginning and end of the string.
 *
 * @param {String} value The string to be trimmed.
 * @returns {String}
 * @export
 */
Aura.Utils.Util.prototype.trim = function(value){};

/**
 * Formats an arbitrary number of arguments into a string by replacing {0}, {1}, ... {n} with the corresponding argument supplied after 'formatString'.
 *
 * @param {String} formatString The string to be formatted.
 * @param {String} arg1...argN The list of arguments to splice into formatString.
 * @returns {String}
 * @export
 */
Aura.Utils.Util.prototype.format=function(formatString,arg1,arg2,argN){};

/**
 * Truncates a string to the given length.
 *
 * @param {String} st The string to be truncated.
 * @param {Number} len The length of characters. Includes the ellipsis if ellipsis is set to true.
 * @param {Boolean} ellipsis If set to true, an ellipsis is added to the truncated string.
 * @param {Boolean} truncateByWord If set to true, checks that no truncation occurs in the middle of a word.
 * @returns {String} The truncated string.
 * @export
 */
Aura.Utils.Util.prototype.truncate = function(st, len, ellipsis, truncateByWord){};

/**
 * Create a function that invokes the given callback after the tolerance period
 * has passed since the last invocation of the function.
 *
 * This is useful to defer responding to a stream of repetetive events until the
 * end of the stream.
 *
 * @param {Function} callback
 *          The function to be run once the tolerance period has passed.
 * @param {Number} toleranceMillis
 *          The tolerance duration in milliseconds.
 * @returns {Function} The function to invoke in order to trigger a start/reset
 *          of the tolerance period.
 * @export
 */
Aura.Utils.Util.prototype.createTimeoutCallback = function(callback, toleranceMillis) {};

/**
 * Adds an event listener to a DOM element.
 *
 * @param {HTMLElement} element The DOM element to which to apply the listener.
 * @param {String} eventName The name of the DOM event, minus the "on" prefix (e.g. "click", "focus", "blur", etc.).
 * @param {Object} handler The JS handler to add.
 * @param {Boolean} useCapture Whether to use event capturing.
 * @param {Number} timeout Optional timeout (in milliseconds) that will delay the handler execution.
 * @returns {Object} Either a function (success) or null (fail)
 * @export
 */
Aura.Utils.Util.prototype.on = (function() {})();

/**
 * Removes an event listener from a DOM element. See also Util.on() a.k.a. $A.util.on()
 *
 * @param {HTMLElement} element The DOM element from which to remove the listener.
 * @param {String} eventName The name of the DOM event, minus the "on" prefix (e.g. "click", "focus", "blur", etc.).
 * @param {Function} listener The JS listener function to remove.
 * @param {Boolean} useCapture Whether to use event capturing.
 * @export
 */
Aura.Utils.Util.prototype.removeOn = function(element, eventName, listener, useCapture) {};

/**
 * Stores the values of a form to a Map object. Values from a checkbox, radio, drop-down list, and textarea
 * are stored in the Map.
 *
 * @param {Object} form
 * @returns {Object} The map containing the values from the form input.
 * @export
 */
Aura.Utils.Util.prototype.formToMap = function(form) {};

/**
 * Gets the selected values from a list of options.
 * Returns a single value if only a single option is selected.
 *
 * @param {Object} select
 * @returns {Object} A list of selected options.
 * @export
 */
Aura.Utils.Util.prototype.getSelectValue = function(select) {};

/**
 * Adds a value to a map with a given key.  If the key already exists, the values are turned into a list.
 * If the value has a dot in it - e.g. "properties.4" - it will be turned into an inner map, with
 * the second part as the inner key.
 *
 * @param {Object} inputMap The input map to be processed.
 * @param {String} key The data key whose value is to be added to the input map.
 * @param {Object} value The value of the data to add to the input map.
 * @export
 */
Aura.Utils.Util.prototype.addValueToMap = function(inputMap, key, value) {};

/**
 * Generates a map of values inside the main input map.  This is used, for example,
 * When input fields have a "." operator, so
 * input name="def.def1"
 * input name="def.def2"
 * get put in the input map under "def", as a map with "def1" and "def2" mapped to their values.
 *
 * @param {Object} inputMap The input map to be processed.
 * @param {String} key The data key whose value is to be added to the input map.
 * @param {Object} value The value of the data to add to the input map.
 * @param {String} subMapKey
 * @export
 */
Aura.Utils.Util.prototype.addMapValueToMap = function(inputMap, key, value, subMapKey) {};

/**
 * Walks up a definition hierarchy to search for a sub definition by qualified name.
 *
 * @param {Object} def
 *          The definition to search
 * @param {String} qname
 *          The qualified name to search for
 * @returns {Boolean} true if qualified name is found in defs hierarchy
 * @export
 */
Aura.Utils.Util.prototype.isSubDef = function(def, qname) {};

/**
 *
 * @description Takes the methods, and properties from one object and assigns them to another.
 * Returns the base object with the members from the child object.
 * This is commonly used to apply a set of configurations to a default set, to get a single set of configuration properties.
 *
 * @example
 * $A.util.apply(Child.prototype, Parent); // Returns a new object inheriting all the methods and properties from Parent.
 *
 * @example
 * $A.util.apply(Child.prototype, { isCool: true }); // Parent would then have a property of child.
 *
 * @example
 * $A.util.apply({ foo: 'bar', diameter: 10}, { diameter: 20, bat: 'man' }, true); //== {foo:'bar', diameter: 20, bat: 'man'}
 *
 * @example
 * $A.util.apply({ foo: 'bar', diameter: 10}, { diameter: 20, bat: 'man' }, false); //== {foo:'bar', diameter: 10, bat: 'man'}
 *
 * @param {Object|Function} baseObject The object that will receive the methods, and properties.
 * @param {Object|Function} members The methods and properties to assign to the baseObject.
 * @param {Boolean} [forceCopy] If the property already exists, should we still copy the member? false by default
 * @param {Boolean} [deepCopy] Should we continue to navigate child objects if we don't overwrite them? false by default
 */
Aura.Utils.Util.prototype.apply = function(/* Object|Function */ baseObject, /* Object|Function*/ members, /* bool */ forceCopy, /* bool */ deepCopy) {};

/**
 * apply() has a bug that it copies values from the prototype.
 * This was found in late 210 a week before FF. Thats to late for a change to apply(), so Kris Gray created
 * this method which does the right thing.
 *
 * We'll fix the bug by using this new method, and in 212, we'll switch apply() to behave like this method.
 * The API and functionality is the same except that it does not copy values from the prototype.
 * Please DO NOT EXPORT THIS METHOD. That makes it much harder to replace in 212.
 */
Aura.Utils.Util.prototype.applyNotFromPrototype = function(/* Object|Function */ baseObject, /* Object|Function*/ members, /* bool */ forceCopy, /* bool */ deepCopy) {};


Aura.Utils.Util.prototype.CAMEL_CASE_TO_HYPHENS_REGEX = /([A-Z])/g;

/**
 * Converts camelCase to hyphens.
 *
 * @param {String} str The string to be converted.
 * @returns {String} The string containing hyphens that replaces the camelCase.
 */
Aura.Utils.Util.prototype.camelCaseToHyphens = function(str) {};

/**
 * Converts hyphens to camelCase.
 *
 * @param {String} str The string to be converted.
 * @returns {String} The string in camelCase.
 */
Aura.Utils.Util.prototype.hyphensToCamelCase = function(str) {};

/**
*  Converts words to camelCase, strips non-alphanumeric characters
*
* */
Aura.Utils.Util.prototype.toCamelCase=function(str) {};

/**
 *
 * @description A map of nodeNames that cannot accept custom data attributes.
 * @private
 */
Aura.Utils.Util.prototype.noData = {
    "embed": true,
    "object": "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000", // flash
    "applet": true,
    "#text": true
};

/**
 *
 * @description Returns whether a given DOM element can accept custom data attributes.
 *
 * @param {HTMLElement} element The element to check for custom data attribute support.
 * @returns {Boolean} Whether element accepts custom data attributes.
 */
Aura.Utils.Util.prototype.acceptsData = function(element) {};

/**
 * Return attributeValue of an element
 *
 * @param {HTMLElement} element The element from which to retrieve data.
 * @param {String} attributeName The name of attribute to look up on element.
 * @export
 */
Aura.Utils.Util.prototype.getElementAttributeValue = function(element,attributeName){};

/**
 *
 * @description Returns a custom data attribute value from a DOM element.
 * For more information on custom data attributes, see http://html5doctor.com/html5-custom-data-attributes/
 * @param {HTMLElement} element The element from which to retrieve data.
 * @param {String} key The data key to look up on element.
 * @export
 */
Aura.Utils.Util.prototype.getDataAttribute = function(element, key) {};

/**
 *
 * @description Sets a custom data attribute value from a DOM element.
 * For more information on custom data attributes, see http://html5doctor.com/html5-custom-data-attributes/
 * @param {HTMLElement} element The element from which to retrieve data.
 * @param {String} key The data key to add to element.
 * @param {String} value The value of the data to add to an element. If value is undefined, the key data attribute will be removed from element.
 * @export
 */
Aura.Utils.Util.prototype.setDataAttribute = function(element, key, value) {};

/**
 * @private
 */
Aura.Utils.Util.prototype.getDataAttributeName = function(key) {};

/**
 * Checks whether a custom data attribute value already exists.
 * @param {HTMLElement} element The element from which to retrieve data.
 * @param {String} key The data key to look up on element.
 * @returns {Boolean} true if element has data attribute
* @export
 */
Aura.Utils.Util.prototype.hasDataAttribute = function(element, key) {};

/**
 * Checks if the object is an HTML element.
 * @param {Object} obj
 * @returns {Boolean} True if the object is an HTMLElement object, or false otherwise.
 */
Aura.Utils.Util.prototype.isHTMLElement = function(obj) {};

/**
 * Checks if the object is a SVG element.
 * @param {Object} obj
 * @returns {Boolean} True if the object is an SVGElement object, or false otherwise.
 */
Aura.Utils.Util.prototype.isSVGElement = function(obj) {};

/**
 * Attach the element to the HTML body
 * @param {DOMElement} element
 * @export
 */
Aura.Utils.Util.prototype.attachToDocumentBody = function(element) {};
/**
* Check for substrings at the end.
* @param {String} fullstr The string to check against.
* @param {String} substr The substring to look for at the end.
* @returns {Boolean} True if fullstr ends with substr.
* @export
*/
Aura.Utils.Util.prototype.stringEndsWith = function(fullstr, substr) {};

/**
 * Creates a new function whith bound arguments.
 * @param {Function} method to bind.
 * @param {Any} bound 'this' instance for the new function's scope.
 * @param {Any} var-args of bound parameters.
 * @returns {Function} a new function that invokes the provided function instance with bound arguments.
 */
if (Function.prototype.bind) {
    Aura.Utils.Util.bind = function(method /*, this, bind arguments*/) {};
} else {
    Aura.Utils.Util.bind = function(method /*, this, bind arguments*/) {};
}

/** @export */
Aura.Utils.Util.prototype.bind = Aura.Utils.Util.bind;

/**
 * Performs a series of 'safe' sequential lookup of nested properies.
 *
 * Example: a safe lookup for "VALUE" in: object: {
 *    first: {
 *       second: [
 *           "VALUE"
 *       ]
 *    }
 * }
 *
 * Can be done via: $A.util.lookup(object, "first", "second", 0);
 * Instead of: object && object.first && object.first.second && object.first.second[0]
 *
 * @param {Object} root object or array to sequentially lookup properties from.
 * @param {String} var-args of string property names.
 * @return {Any} the looked-up property or undefined if any properties along the way were not found.
 * @export
 */
Aura.Utils.Util.prototype.lookup = function(object /*, var-args of arrays*/) {};


/**
 * Does an in-place merge of any number of array into the first.
 * @param {Array} array to receive the elements of subsequent arrays.
 * @param {Array} var-args of arrays that will have their elements copied into the first.
 * @returns {Array} the first array (which has been modified in-place).
 * @export
 */
Aura.Utils.Util.prototype.merge = function(first /*, var-args of arrays*/) {};

/** forEach: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.forEach) {
    Aura.Utils.Util.forEach = function(array, method, that) {};
} else {
    /**
     * Runs a function over each element in an array.
     * @param {Array} array to loop over.
     * @param {Function} method to call for each element.
     * @param {Any} the 'this' instance inside the scope of provided method.
     */
    Aura.Utils.Util.forEach = function(array, method, that) {};
}

/** @export */
Aura.Utils.Util.prototype.forEach = Aura.Utils.Util.forEach;

/** map: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.map) {
    Aura.Utils.Util.map = function(array, method, that) {};
} else {
    /**
     * Returns an array containing the return value of the provided function over every element of the input array.
     * @param {Array} array to loop over.
     * @param {Function} tranforms an element from the input array to an element in the output array.
     * @param {Any} the 'this' instance inside the scope of provided transformation method.
     * @returns {Array} where every element is a result of the transformation function
     * applied to the element (at the same index) from the input array.
     */
    Aura.Utils.Util.map = function(array, method, that) {};
}
/** @export */
Aura.Utils.Util.prototype.map = Aura.Utils.Util.map;

/** reduce: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.reduce) {
    Aura.Utils.Util.reduce = function(array, method, initial) {};
} else {
    /**
     * Loops over an array, calling a function that provides the returned result of calling the function on the
     * previous element.
     * @param {Array} array to loop over.
     * @param {Function} reduction method that takes the resturned result from the previous call, the current element from
     * the input array and index.
     * @param {Any} the initial object passed to the first element in the array's reduction method.
     * @returns {Any} the final value returned from calling the reduction method on the last element.
     */
    Aura.Utils.Util.reduce = function(array, method, initial) {};
}

/** @export */
Aura.Utils.Util.prototype.reduce = Aura.Utils.Util.reduce;

/** every: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.every) {
    Aura.Utils.Util.every = function(array, predicate, that) {};
} else {
    /**
     * Loops over an array, calling a function that returns some boolean. Returns true if all calls return a truthy result.
     * @param {Array} array to loop over.
     * @param {Function} predicate that returns a boolean result based on the current array element.
     * @param {Any} the 'this' instance inside the scope of provided transformation method.
     * @returns {Boolean} true if all elements of the array satisfy the predicate.
     */
    Aura.Utils.Util.every = function(array, predicate, that) {};
}

/** @export */
Aura.Utils.Util.prototype.every = Aura.Utils.Util.every;

/** some: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.some) {
    Aura.Utils.Util.some = function(array, predicate, that) {};
} else {
    /**
     * Loops over an array, calling a function that returns some boolean. Returns true if any calls return a truthy result.
     * @param {Array} array to loop over.
     * @param {Function} predicate that returns a boolean result based on the current array element.
     * @param {Any} the 'this' instance inside the scope of provided transformation method.
     * @returns {Boolean} true if any of the elements of the array satisfy the predicate.
     */
    Aura.Utils.Util.some = function(array, predicate, that) {};
}

/** @export */
Aura.Utils.Util.prototype.some = Aura.Utils.Util.some;

/** filter: see documentation below (attached to last definition so that it is picked up for doc generation). */
if (Array.prototype.filter) {
    Aura.Utils.Util.filter = function(array, predicate, that) {};
} else {
    /**
     * Loops over an array, constructing a new array with the elements that pass the filter predicate.
     * @param {Function} predicate that returns a boolean result based on the current array element the result of which
     *                   indicates whether the element will be returned in the filter result array.
     * @param {Any} the 'this' instance inside the scope of provided predicate.
     * @returns {Array} ordered array of elements that pass the predicate.
     */
    Aura.Utils.Util.filter = function(array, predicate, that) {};
}

/** @export */
Aura.Utils.Util.prototype.filter = Aura.Utils.Util.filter;

/**
 * Schedules the specified component to be asynchronously destroyed.
 * @param {Component} cmp
 *          The component to be destroyed.
 * @private
 */
Aura.Utils.Util.prototype.destroyAsync = function(cmp) {};

/**
 * Returns whether "instance" is, directly or indirectly, an instance of
 * "constructor."  An object is indirectly an instance if derivePrototypeFrom was
 * used to make the child type derive from the parent type.
 *
 * JavaScript's instanceof operator is not used as it doesn't understand
 * type inheritance. Using this method would avoid the need for child.prototype to be
 * an instance of parent; we also avoid having "unbound" instances.
 *
 * @param instance The object to test
 * @param constructor  The object to test against
 * @returns {Boolean} Returns true if instance is an instance of constructor.
 * @export
 */
Aura.Utils.Util.prototype.instanceOf = function(instance, constructor) {};

/**
 * Destroys any components currently in the trashcan.
 * @private
 */
Aura.Utils.Util.prototype.emptyComponentTrash = function() {};

/**
 * Determines if an element is either a descendant of, or the same as, another element in the DOM tree.
 * Both arguments to this function must be of type HTMLElement or SVGElement.
 *
 * @param {(HTMLElement|SVGElement)} container The element you think is the outermost container.
 * @param {(HTMLElement|SVGElement)} element The element you think is buried inside the container.
 * @returns {Boolean} Returns true if 'element' is indeed inside 'container', false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.contains = function(container, element) {};


/**
 * Simple event squasher.
 *
 * @param {UIEvent} event the DOM event to squash
 * @param {Boolean} preventDefault if preventDefault() should also be called
 * @export
 */
Aura.Utils.Util.prototype.squash = function(event, preventDefault) {};

/**
 * Strip off html tags from html codes.
 *
 * @param {String} input the input html codes
 * @param {Array} tags the html tag names to be removed
 * @return {String} an output string without those specified tags
 * @export
 */
Aura.Utils.Util.prototype.stripTags = function(input, tags) {};


/**
 * Simple function to get client viewport dimensions. If neither window.innerWidth
 * nor document.body.clientWidth is supported by the client, returns "0" for
 * both width and height.
 *
 * @return {Object} JS object with the fields "width" and "height"
 * @export
 */
Aura.Utils.Util.prototype.getWindowSize = function() {};

/**
 * Checks if the object is an Aura component.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is a component, or return false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isComponent = function(obj) {};

/**
 * Checks if the object is an Aura value expression.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is an expression, or false otherwise.
 * @export
 */
Aura.Utils.Util.prototype.isExpression = function (obj) {};

/**
 * Checks if the object is an Aura value object.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is an Aura value (PropertyReferenceValue, FunctionCallValue, PassthroughValue).
 * @export
 */
Aura.Utils.Util.prototype.isValue = function(obj) {};

/**
 * Checks if the object is an Aura action object.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is an Aura Action.
 * @export
 */
Aura.Utils.Util.prototype.isAction = function(obj) {};

/**
 * Checks if the object is an Aura Event object.
 *
 * @param {Object} obj The object to check for.
 * @returns {Boolean} True if the object type is an Aura Event.
 * @export
 */
Aura.Utils.Util.prototype.isEvent = function(obj) {};

/**
 * Checks if touch events are supported. Cache the result, it shouldn't change.
 *
 * @returns {Boolean} True if touch events are supported.
 * @export
*/
Aura.Utils.Util.prototype.supportsTouchEvents = function() {};

/**
 * Estimate the size of an object or primitive in bytes.
 *
 * @param {Object} item The object or primitive whose size to estimate.
 * @return {Number} The estimated size of the item in bytes.
 * @export
 */
Aura.Utils.Util.prototype.estimateSize = function(obj) {};


/**
 * Convert collection to a true array.
 * When dealing with a NodeList, sometimes you'll need it to actually be an array to properly deal with it.
 * Cannot always use Array.prototype.slice.call(), since it doesn't work in IE6/7/8 on NodeLists.
 * @returns An empty array if you pass a null or undefined value to collection.
 */
Aura.Utils.Util.prototype.toArray = function(collection) {};

/** @export */
Aura.Utils.Util.prototype.setText = function(node, text) {};

/**
 * Posts message to the provided window. This was done to workaround an issue where browser sets
 * event.source to be safeEvalWorker window (see W-3443540).
 * NOTE: safeEvalWorker has been removed.
 * @param {Window} targetWindow The destination window for the message
 * @param {Array} argsArray list of arguments to be posted
 * @export
 */
Aura.Utils.Util.prototype.postMessage = function(targetWindow, argsArray){};

/**
 * Get a string representation of component hierarchy by calling getOwner and walk up the component tree.
 * @param {component} leaf component to walk up the hierarchy
 * @private
 */
Aura.Utils.Util.prototype.getComponentHierarchy = function(component){};

/**
 * eval throwing an error to check whether sourceURL is supported.
 * @private
 */
Aura.Utils.Util.prototype.hasSourceURL = function() {};

/**
 * Returns a hash for a passed in string.
 * Replicates Java's hashCode method (https://docs.oracle.com/javase/7/docs/api/java/lang/String.html#hashCode()).
 * @param {String} a string to be hashed.
 * @returns {Number} a hashed representation of the passed in string.
 * @private
 */
Aura.Utils.Util.prototype.getHashCode = function(value) {};

//#if {"excludeModes" : ["PRODUCTION", "PRODUCTIONDEBUG"]}
    /**
     * Gets the aura debug tool component whether in an iframe or not.
     * @returns {Object} The debug tool component.
     * @export
     */
    Aura.Utils.Util.prototype.getDebugToolComponent = function(){};

    /**
     * Gets the aura instance of debug tool which has been opened in a child window
     *
     * @returns {Object} Aura instance
     * @export
     */
    Aura.Utils.Util.prototype.getDebugToolsAuraInstance = function(){};

    /**
     * Set the aura debug tool handle when opened in a popup.
     * @export
     */
    Aura.Utils.Util.prototype.setDebugToolWindow = function(debugWindow){};

    /**
     * Grab windows url, if debug tool is a child window get url of parent
     *
     * @returns {String} URL of the window
     * @export
     */
    Aura.Utils.Util.prototype.getUrl = function(){};

    /**
     * Get the text content of a DOM node. Tries <code>textContent</code> followed by
     * <code>innerText</code>, followed by <code>nodeValue</code> to take browser differences into account.
     * @param {Node} node The node to get the text content from
     * @returns {String} The text content of the DOM node or empty string if unable to extract text
     * @export
     */
    Aura.Utils.Util.prototype.getText = function(node) {};

    /**
     * Loads a JavaScript resource.
     * @param {String} url The URL of the JavaScript resource to load.
     * @param {Function} callback Callback function that is invoked when the resource is loaded.
     * @export
     */
    Aura.Utils.Util.prototype.includeScript = function(url, callback) {};
//#end
