/*
 * Copyright (C) 2013 salesforce.com, inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @description A value provider that resolves against a few primary providers first, then falls back on a component.
 * @constructor
 * @protected
 * @export
 */
function PassthroughValue(primaryProviders, component) {}

/**
 * Since PassthroughValue can have its own set of values that can be listen for changes,
 * it needs it's own value change handler logic. Essentially you should be able to treat
 * it like a component for change events. It does not mark dirty though, since a passthrough
 * does have anything to rerender, marking dirty is the responsibility of the referencing components.
 * @export
 */
PassthroughValue.prototype.addValueHandler = function(config) {};

PassthroughValue.prototype.addChangeHandler=function(config){};


/**
 * Delegates de-indexing logic to the wrapped value provider.
 * Likely delegating to a wrapped component.
 * @export
 */
PassthroughValue.prototype.deIndex = function () {};

 /**
 * Fires handlers registered for the specified key when the value changes
 * @export
 */
PassthroughValue.prototype.fireChangeEvent = function(key, oldValue, value, index) {};

/**
 * Returns the primary providers associated with the given key or the Component.
 * @param {String} key The data key to look up on the primary providers.
 * @export
 */
PassthroughValue.prototype.get = function(key) {};

/**
 * Returns the Component.
 * @export
 */
PassthroughValue.prototype.getComponent = function() {};

/**
 * Gets all the keys for the additional providers specified in the passthrough value.
 * @export
 * @return Array of keys
 */
PassthroughValue.prototype.getPrimaryProviderKeys = function() {};

/**
 * Get the definition of the valueProvider component. 
 * If the immediate value provider for this passthrough value is another passthrough value
 * it will resolve up the chain till it finds a valid component.
 */
PassthroughValue.prototype.getDef=function(){};

/**
 * Passthrough's have extra providers that can reference other items of data.
 * If it's raw data, no problem. If it's another reference, you may want to
 * expand that reference. {row.value} could expand into {v.item.0.value} if row
 * is at index 0.
 * @param {String} expression The key to reference on the component, which will get expanded into the reference you were looking for.
 */
PassthroughValue.prototype.getExpression = function(expression) {};

/**
 * Returns a reference to a key on the the primary provider or the Component.
 * @param {String} key The data key for which to return a reference.
 * @export
 */
PassthroughValue.prototype.getReference = function(key) {};

/**
 * Delegates indexing logic to the wrapped value provider.
 * Likely delegating to a wrapped component.
 * @export
 */
PassthroughValue.prototype.index = function () {};

/**
 * Removes a handler for the specified type of event. Currently only supports
 * 'change'.
 * @export
 */
PassthroughValue.prototype.removeValueHandler = function(config) {};

/**
 * Sets the value of the primary providers associated value.
 * @param {String} key The data key to look up on the primary providers.
 * @param {Object} v The value to be set.
 * @export
 */
PassthroughValue.prototype.set = function(key, value, ignoreChanges) {};

/**
 * Returns true if the referenced component has not been destroyed.
 */
PassthroughValue.prototype.isValid = function() {};
