/*
 * Copyright (c) 2016 Scott Wells, All rights reserved
 */

// Stub interfaces for the JavaScript Salesforce1 API taken from:
// https://developer.salesforce.com/docs/atlas.en-us.salesforce1.meta/salesforce1/salesforce1_dev_jsapi_sforce_one.htm
// TODO: What else?

// sforce namespace
sforce = {};
sforce.prototype = {
    // Properties
    one: null
};

// sforce.one namespace
sforce = {};
/**
 * The Salesforce App Cloud includes an event mechanism for navigation. This is exposed in Visualforce as a JavaScript
 * object called <code>sforce.one</code>. It's available in any page that appears in Salesforce1. The sforce.one object
 * provides the following functions. Reference the function using dotted notation from the sforce.one object. For
 * example:
 *
 * <pre>
 * sforce.one.navigateToSObject(recordId, view).
 * </pre>
 *
 * @type {sforce.one}
 * @const
 * @public
 */
sforce.one = {};
sforce.one.prototype = {
    // Methods

    /**
     * Navigates to the previous state that's saved in the sforce.one history. It's equivalent to clicking a browser's
     * Back button.
     *
     * @param refresh <code>refresh</code> is optional. By default, the page doesn't refresh. Pass true to refresh the
     *     page if possible.
     *
     * @public
     */
    back: function (refresh) {},

    /**
     * Navigates to an sObject record, specified by recordId. This record "home" has several views, which in
     * Salesforce1 are available as slides that the user can swipe between.
     *
     * @param recordId the record ID
     * @param view <code>view</code> is optional and defaults to <code>detail</code>. <code>view</code> specifies the
     *     slide within record home to display initially. The possible values are as follows:
     *     <ul>
     *         <li><code>detail</code>: the record detail slide</li>
     *         <li><code>chatter</code>: the Chatter slide</li>
     *         <li><code>related</code>: the view of related slide</li>
     *     </ul>
     *
     * @public
     */
    navigateToSObject: function (recordId, view) {},

    /**
     * Navigates to the specified URL.
     * Relative and absolute URLs are supported. Relative URLs are relative to the one.app domain, and retain
     * navigation history. External URLs—that is, URLs that are outside the Salesforce1 <code>one.app</code>
     * domain—open in a separate browser window.
     *
     * <b>Note</b> Depending on the user's device plaform, device settings, version of the Salesforce1 downloadable
     * app, and authentication requirements for the external URL being opened, the separate browser window might
     * require authentication or re-authentication.
     *
     * Use relative URLs to navigate to different screens within your app. Use external URLs to allow the user to
     * access a different site or app, where they can take actions that don't need to be preserved in your app. To
     * return to your app, the separate window that's opened by an external URL must be closed when the user is
     * finished with the other app. The new window has a separate history from your app, and this history is discarded
     * when the window is closed. This also means that the user can't click a Back button to go back to your app; the
     * user must close the new window.
     *
     * <code>mailto:</code>, <code>tel:</code>, <code>geo:</code>, and other URL schemes are supported for launching
     * external apps and attempt to "do the right thing." However, support varies by mobile platform and device.
     * <code>mailto:</code> and <code>tel:</code> are reliable, but we recommend that you test any other URLs on a
     * range of expected devices.
     *
     * <b>Note</b> Only standard URL schemes are supported by <code>navigateToURL</code>. To access custom schemes, use
     * <code>window.location</code> instead.
     *
     * @param url the URL
     * @param isRedirect <code>isredirect</code> is optional and defaults to <code>false</code>. Set it to
     *     <code>true</code> to indicate that the new URL should replace the current one in the navigation history.
     *
     * @public
     */
    navigateToURL: function (url, isRedirect) {},

    /**
     * Navigates to the feed of the specified type, scoped to the <code>subjectId</code>. For some feed types, the
     * <code>subjectId</code> is required but ignored. For those feed types, pass the current user's ID as the
     * <code>subjectId</code>.
     *
     * <code>type</code> is the feed type. The possible values are as follows:
     * <ul>
     * <li><code>BOOKMARKS</code>: Contains all feed items saved as bookmarks by the context user. Pass the current
     * user's ID as the <code>subjectId</code>.</li>
     * <li><code>COMPANY</code>: Contains all feed items except feed items of type TrackedChange. To see the feed item,
     * the user must have sharing access to its parent. Pass the current user's ID as the <code>subjectId</code>.</li>
     * <li><code>FILES</code>: Contains all feed items that contain files posted by people or groups that the context
     * user follows. Pass the current user's ID as the <code>subjectId</code>.</li>
     * <li><code>GROUPS</code>: Contains all feed items from all groups the context user either owns or is a member of.
     * Pass the current user's ID as the <code>subjectId</code>.</li>
     * <li><code>NEWS</code>: Contains all updates for people the context user follows, groups the user is a member of,
     * and files and records the user is following. Also contains all updates for records whose parent is the context
     * user and every feed item and comment that mentions the context user or that mentions a group the context user is
     * a member of. Pass the current user's ID as the <code>subjectId</code>.</li>
     * <li><code>PEOPLE</code>: Contains all feed items posted by all people the context user follows. Pass the current
     * user's ID as the <code>subjectId</code>.</li>
     * <li><code>RECORD</code>: Contains all feed items whose parent is a specified record, which could be a group,
     * user, object, file, or any other standard or custom object. When the record is a group, the feed also contains
     * feed items that mention the group. When the record is a user, the feed contains only feed items on that user.
     * You can get another user's record feed. Pass the record's ID as the <code>subjectId</code>.</li>
     * <li><code>TO</code>: Contains all feed items with mentions of the context user, feed items the context user
     * commented on, and feed items created by the context user that are commented on. Pass the current user's ID as
     * the <code>subjectId</code>.</li>
     * <li><code>TOPICS</code>: Contains all feed items that include the specified topic. Pass the topic's ID as the
     * <code>subjectId</code>. This value is supported for the Salesforce1 mobile browser app only. Topics aren't
     * available in the Salesforce1 downloadable apps.</li>
     * </ul>
     *
     * @param subjectId the subject ID
     * @param type the feed type
     *
     * @public
     */
    navigateToFeed: function (subjectId, type) {},

    /**
     * Navigates to the specific feed item, <code>feedItemId</code>, and any associated comments.
     *
     * @param feedItemId the feed item ID
     *
     * @public
     */
    navigateToFeedItemDetail: function (feedItemId) {},

    /**
     * Navigates to a related list for the <code>parentRecordId</code>. For example, to display a related list for a
     * Warehouse object, the <code>parentRecordId</code> is <code>Warehouse__c.Id</code>.
     *
     * @param relatedListId the API name or ID of the related list to display.
     * @param parentRecordId the parent record ID
     *
     * @public
     */
    navigateToRelatedList: function (relatedListId, parentRecordId) {},

    /**
     * Navigates to the list view that's specified by the <code>listViewId</code>, which is the ID of the list view to
     * be displayed. <code>listViewName</code> sets the title for the list view. It doesn't need to match the actual
     * name that's saved for the list view. To use the saved name, set <code>listViewName</code> to <code>null</code>.
     *
     * Set scope to the name of the sObject in the view, for example, "Account" or "MyObject__c".
     *
     * @param listViewId the list view ID
     * @param listViewName the list view name
     * @param scope the name of the sObject in the view
     *
     * @public
     */
    navigateToList: function (listViewId, listViewName, scope) {},

    /**
     * Opens the page to create a new record for the specified <code>entityName</code>, for example, "Account" or
     * "MyObject__c".
     *
     * @param entityName the entity name
     * @param recordTypeId <code>recordTypeId</code> is optional and, if provided, specifies the record type for the
     *     created object.
     *
     * @public
     */
    createRecord: function (entityName, recordTypeId) {},

    /**
     * Opens the page to edit the record specified by <code>recordId</code>.
     *
     * @param recordId the record ID
     *
     * @public
     */
    editRecord: function (recordId) {}
};