/*
 * Copyright (c) 2016 Scott Wells, All rights reserved
 */

// Stub interfaces for the JavaScript Visualforce API taken from:
// https://developer.salesforce.com/docs/atlas.en-us.pages.meta/apexcode/pages_js_remoting_summary.htm
// TODO: https://developer.salesforce.com/docs/atlas.en-us.pages.meta/apexcode/pages_remote_objects.htm

// UITheme namespace
UITheme = {
    // Methods

    /**
     * Returns a string that indicates the current user's interface theme.
     *
     * @type {String}
     * @public
     */
    getUITheme: function () {}
};

// Visualforce namespace
Visualforce = {};
Visualforce.prototype = {
    // Properties
    remoting: null
};

// Visualforce.remoting namespace
Visualforce.remoting = {};
Visualforce.remoting.prototype = {
    // Properties

    /**
     * Whether to group requests executed close to each other in time into a single request. The default is
     * <code>true</code>.
     *
     * JavaScript remoting optimizes requests that are executed close to each other in time and groups the calls into a
     * single request. This buffering improve the efficiency of the overall request-and-response cycle, but sometimes
     * it's useful to ensure all requests execute independently.
     *
     * @type {Boolean}
     * @public
     */
    buffer: false,

    /**
     * Whether to escape the Apex method's response. The default is <code>true</code>.
     *
     * @type {Boolean}
     * @public
     */
    escape: false,

    /**
     * The timeout for the request, in milliseconds. The default is 30,000 (30 seconds). The maximum is 120,000 (120
     * seconds, or 2 minutes).
     *
     * @type {int}
     * @public
     */
    timeout: 0,

    Manager: null,
    Event: null
};

/**
 * Provides access to
 * {@link https://developer.salesforce.com/docs/atlas.en-us.pages.meta/pages/pages_js_remoting.htm|JavaScript
 * Remoting}.
 *
 * @type {Visualforce.remoting.Manager}
 * @const
 * @public
 */
Visualforce.remoting.Manager = {};
Visualforce.remoting.Manager.prototype = {
    // Methods

    /**
     * Invokes an Apex method using JavaScript Remoting.
     *
     * @param method The name of the Apex method you're calling.
     * @param parameters A comma-separated list of parameters that your method takes.
     * @param callback The name of the JavaScript function that will handle the response from the controller. You can
     *     also declare an anonymous function inline. <code>callbackFunction</code> receives the status of the method
     *     call and the result as parameters.
     * @param configuration Configures the handling of the remote call and response. Use this to change the behavior of
     *     a remoting call, such as whether or not to escape the Apex method's response. Additional details can be
     *     found
     *     {@link https://developer.salesforce.com/docs/atlas.en-us.pages.meta/pages/pages_js_remoting_configuring_request.htm|here}.
     *
     * @public
     */
    invokeAction: function (method, parameters, callback, configuration) {}
};

/**
 * Your callback function will receive as parameters an event object representing the status of the remote call, and
 * the result object returned by the remote Apex method. Your function can update information and user interface
 * elements on the page based on the data returned.
 *
 * The event object provides values that let you act upon the success or failure of the remote call.
 *
 * @type {Visualforce.remoting.Event}
 * @const
 * @public
 */
Visualforce.remoting.Event = {};
Visualforce.remoting.Event.prototype = {
    // Properties

    /**
     * <code>true</code> on success, <code>false</code> on error.
     *
     * @type {Boolean}
     * @public
     */
    status: null,

    /**
     * The type of the response: <code>rpc</code> for a successful call, <code>exception</code> if the remote method
     * threw an exception, and so on.
     *
     * @type {String}
     * @public
     */
    type: null,

    /**
     * Contains any error message that is returned.
     *
     * @type {String}
     * @public
     */
    message: null,

    /**
     * Contains the Apex stack trace, if one was generated by the remote method.
     *
     * @type {String}
     * @public
     */
    where: null
};
